@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2025 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2026 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -440,7 +440,7 @@ void configureCommandChannel() {
     }
 }
 
-/// @brief Process a DHCPv6 confguration and return an answer stating if the
+/// @brief Process a DHCPv6 configuration and return an answer stating if the
 /// configuration is valid, or specifying details about the error otherwise.
 ///
 /// @param config_set the configuration being processed
@@ -452,6 +452,8 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
     // for option definitions. This is equivalent to committing empty container.
     LibDHCP::setRuntimeOptionDefs(OptionDefSpaceContainer());
 
+    IfaceMgr::instance().detectIfaces(true);
+
     // Answer will hold the result.
     ConstElementPtr answer;
 
