@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2025 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2026 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -768,7 +768,8 @@ public:
         // Note that when comparing addresses, we compare the textual
         // representation. IOAddress does not support being streamed to
         // an ostream, which means it can't be used in EXPECT_EQ.
-        EXPECT_TRUE(subnet_->inPool(type, addr->getAddress()));
+        EXPECT_TRUE(subnet_->inPool(type, addr->getAddress()))
+                    << "address:" << addr->getAddress().toText();
         EXPECT_EQ(expected_addr.toText(), addr->getAddress().toText());
         if (subnet_->getPreferred().getMin() != subnet_->getPreferred().getMax()) {
             EXPECT_LE(subnet_->getPreferred().getMin(), addr->getPreferred());
