@@ -25,6 +25,7 @@
 using namespace isc;
 using namespace isc::asiolink;
 using namespace isc::data;
+using namespace isc::tcp;
 using namespace isc::util;
 using namespace std;
 using namespace std::chrono;
@@ -55,21 +56,13 @@ exchangeRCtoText(const int rc) {
     }
 }
 
-Exchange::Exchange(const asiolink::IOServicePtr io_service,
-                   const MessagePtr& request,
+Exchange::Exchange(const MessagePtr& request,
                    unsigned maxretries,
                    const Servers& servers,
                    Handler handler)
-    : identifier_(""), io_service_(io_service), sync_(false),
-      started_(false), terminated_(false), rc_(ERROR_RC),
-      start_time_(std::chrono::steady_clock().now()),
-      socket_(), ep_(), timer_(), server_(), idx_(0),
-      request_(request), sent_(), received_(), buffer_(), size_(0),
-      retries_(0), maxretries_(maxretries), servers_(servers),
-      postponed_(), handler_(handler), mutex_(new std::mutex()) {
-    if (!io_service) {
-        isc_throw(BadValue, "null IO service");
-    }
+    : identifier_(""), sync_(false), rc_(ERROR_RC),
+      request_(request), sent_(), received_(),
+      maxretries_(maxretries), servers_(servers), handler_(handler) {
     if (!request) {
         isc_throw(BadValue, "null request");
     }
@@ -85,14 +78,9 @@ Exchange::Exchange(const asiolink::IOServicePtr io_service,
 Exchange::Exchange(const MessagePtr& request,
                    unsigned maxretries,
                    const Servers& servers)
-    : identifier_(""), io_service_(new IOService()), sync_(true),
-      started_(false), terminated_(false), rc_(ERROR_RC),
-      start_time_(std::chrono::steady_clock().now()),
-      socket_(), ep_(), timer_(), server_(), idx_(0),
-      request_(request), sent_(), received_(), buffer_(), size_(0),
-      retries_(0), maxretries_(maxretries), servers_(servers), postponed_(),
-      handler_(), mutex_(new std::mutex()) {
-
+    : identifier_(""), sync_(true), rc_(ERROR_RC),
+      request_(request), sent_(), received_(),
+      maxretries_(maxretries), servers_(servers), handler_() {
     if (!request) {
         isc_throw(BadValue, "null request");
     }
@@ -102,7 +90,58 @@ Exchange::Exchange(const MessagePtr& request,
     createIdentifier();
 }
 
-Exchange::~Exchange() {
+ExchangePtr
+Exchange::create(const asiolink::IOServicePtr io_service,
+                 const MessagePtr& request,
+                 unsigned maxretries,
+                 const Servers& servers,
+                 Handler handler,
+                 RadiusProtocol protocol) {
+    if (protocol == PW_PROTO_UDP) {
+        return (UdpExchangePtr(new UdpExchange(io_service, request, maxretries,
+                                               servers, handler)));
+    } else {
+        return (TcpExchangePtr(new TcpExchange(request, maxretries,
+                                               servers, handler)));
+    }
+}
+
+ExchangePtr
+Exchange::create(const MessagePtr& request,
+                 unsigned maxretries,
+                 const Servers& servers) {
+    return (UdpExchangePtr(new UdpExchange(request, maxretries, servers)));
+}
+
+UdpExchange::UdpExchange(const asiolink::IOServicePtr io_service,
+                         const MessagePtr& request,
+                         unsigned maxretries,
+                         const Servers& servers,
+                         Handler handler)
+    : Exchange(request, maxretries, servers, handler),
+      io_service_(io_service), started_(false), terminated_(false),
+      start_time_(std::chrono::steady_clock().now()),
+      socket_(), ep_(), timer_(), server_(), idx_(0),
+      buffer_(), size_(0), retries_(0), postponed_(),
+      mutex_(new std::mutex()) {
+    if (!io_service) {
+        isc_throw(BadValue, "null IO service");
+    }
+}
+
+UdpExchange::UdpExchange(const MessagePtr& request,
+                         unsigned maxretries,
+                         const Servers& servers)
+    : Exchange(request, maxretries, servers),
+      io_service_(new IOService()),
+      started_(false), terminated_(false),
+      start_time_(std::chrono::steady_clock().now()),
+      socket_(), ep_(), timer_(), server_(), idx_(0),
+      buffer_(), size_(0), retries_(0), postponed_(),
+      mutex_(new std::mutex()) {
+}
+
+UdpExchange::~UdpExchange() {
     MultiThreadingLock lock(*mutex_);
     shutdownInternal();
     timer_.reset();
@@ -126,7 +165,6 @@ Exchange::createIdentifier() {
     rs << hex << setfill('0') << setw(8) << ri;
     identifier_ = rs.str();
 }
-
 void
 Exchange::logReplyMessages() const {
     if (!received_) {
@@ -148,7 +186,7 @@ Exchange::logReplyMessages() const {
 }
 
 void
-Exchange::start() {
+UdpExchange::start() {
     MultiThreadingLock lock(*mutex_);
 
     if (started_) {
@@ -156,8 +194,8 @@ Exchange::start() {
     } else {
         started_ = true;
     }
-    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_EXCHANGE_START)
-      .arg(identifier_);
+    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_UDP_EXCHANGE_START)
+        .arg(identifier_);
 
     open();
 
@@ -168,21 +206,22 @@ Exchange::start() {
         // Done.
         io_service_.reset();
 
-        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_EXCHANGE_SYNC_RETURN)
+        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                  RADIUS_UDP_EXCHANGE_SYNC_RETURN)
             .arg(identifier_)
             .arg(rc_);
     }
 }
 
 void
-Exchange::shutdown() {
+UdpExchange::shutdown() {
     // Avoid multiple terminations.
     MultiThreadingLock lock(*mutex_);
     shutdownInternal();
 }
 
 void
-Exchange::shutdownInternal() {
+UdpExchange::shutdownInternal() {
     if (terminated_) {
         return;
     } else {
@@ -205,8 +244,9 @@ Exchange::shutdownInternal() {
 }
 
 void
-Exchange::buildRequest() {
-    if (!server_) {
+Exchange::buildRequest(const ServerPtr& server,
+                       std::chrono::steady_clock::time_point start_time) {
+    if (!server) {
         isc_throw(Unexpected, "no server");
     }
 
@@ -225,7 +265,7 @@ Exchange::buildRequest() {
     }
 
     // Set the secret.
-    sent_->setSecret(server_->getSecret());
+    sent_->setSecret(server->getSecret());
 
     // Get attributes.
     AttributesPtr attrs = sent_->getAttributes();
@@ -237,14 +277,14 @@ Exchange::buildRequest() {
     // Add Acct-Delay-Time to Accounting-Request message.
     if ((sent_->getCode() == PW_ACCOUNTING_REQUEST) &&
         (attrs->count(PW_ACCT_DELAY_TIME) == 0)) {
-        auto delta = steady_clock().now() - start_time_;
+        auto delta = steady_clock().now() - start_time;
         seconds secs = duration_cast<seconds>(delta);
         attrs->add(Attribute::fromInt(PW_ACCT_DELAY_TIME,
                                       static_cast<uint32_t>(secs.count())));
     }
 
     // Add NAS-IP[v6]-Address with the local address.
-    IOAddress local_addr = server_->getLocalAddress();
+    IOAddress local_addr = server->getLocalAddress();
     short family = local_addr.getFamily();
     if (family == AF_INET) {
         if (attrs->count(PW_NAS_IP_ADDRESS) == 0) {
@@ -257,8 +297,9 @@ Exchange::buildRequest() {
         }
     }
 
-    // Add Message-Authenticator to Status-Server message.
-    if ((sent_->getCode() == PW_STATUS_SERVER) &&
+    // Add Message-Authenticator to Status-Server message or when wanted.
+    if ((RadiusImpl::instance().use_message_authenticator_ ||
+         (sent_->getCode() == PW_STATUS_SERVER)) &&
         (attrs->count(PW_MESSAGE_AUTHENTICATOR) == 0)) {
         const vector<uint8_t> zero(AUTH_VECTOR_LEN);
         // The FreeRADIUS server prefers to get it first.
@@ -271,7 +312,17 @@ Exchange::buildRequest() {
 }
 
 void
-Exchange::open() {
+UdpExchange::buildRequest() {
+    Exchange::buildRequest(server_, start_time_);
+}
+
+void
+TcpExchange::buildRequest() {
+    Exchange::buildRequest(server_, start_time_);
+}
+
+void
+UdpExchange::open() {
     if (RadiusImpl::shutdown_) {
         shutdownInternal();
         return;
@@ -299,14 +350,14 @@ Exchange::open() {
                 (server_->getDeadtimeEnd() > start_time_)) {
                 postponed_.push_back(idx_);
                 ++idx_;
-                io_service_->post(std::bind(&Exchange::openNext,
+                io_service_->post(std::bind(&UdpExchange::openNext,
                                             shared_from_this()));
                 return;
             }
         } else {
             // Second pass: try postponed servers.
             if (postponed_.empty()) {
-                io_service_->post(std::bind(&Exchange::terminate,
+                io_service_->post(std::bind(&UdpExchange::terminate,
                                             shared_from_this()));
                 return;
             }
@@ -357,7 +408,7 @@ Exchange::open() {
             size_ = buffer_.size();
 
             LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-                      RADIUS_EXCHANGE_SEND_NEW)
+                      RADIUS_UDP_EXCHANGE_SEND_NEW)
                 .arg(identifier_)
                 .arg(buffer_.size())
                 .arg(idx_)
@@ -365,13 +416,13 @@ Exchange::open() {
                 .arg(ep_->getPort());
 
             socket_->asyncSend(&buffer_[0], buffer_.size(), ep_.get(),
-                               std::bind(&Exchange::sentHandler,
+                               std::bind(&UdpExchange::sentHandler,
                                          shared_from_this(),
                                          ph::_1,   // error_code.
                                          ph::_2)); // size.
             return;
         } catch (const Exception& exc) {
-            LOG_ERROR(radius_logger, RADIUS_EXCHANGE_OPEN_FAILED)
+            LOG_ERROR(radius_logger, RADIUS_UDP_EXCHANGE_OPEN_FAILED)
                 .arg(identifier_)
                 .arg(exc.what());
             cancelTimer();
@@ -380,7 +431,7 @@ Exchange::open() {
                 socket_->close();
                 socket_.reset();
             }
-            io_service_->post(std::bind(&Exchange::openNext,
+            io_service_->post(std::bind(&UdpExchange::openNext,
                                         shared_from_this()));
             return;
         }
@@ -402,7 +453,7 @@ Exchange::open() {
         if (idx_ == servers_.size()) {
             // Postponed servers are exhausted.
             if (postponed_.size() < 2) {
-                io_service_->post(std::bind(&Exchange::terminate,
+                io_service_->post(std::bind(&UdpExchange::terminate,
                                             shared_from_this()));
                 return;
             }
@@ -412,13 +463,13 @@ Exchange::open() {
             // Try next server.
             ++idx_;
             if ((idx_ == servers_.size()) && (postponed_.empty())) {
-                io_service_->post(std::bind(&Exchange::terminate,
+                io_service_->post(std::bind(&UdpExchange::terminate,
                                             shared_from_this()));
                 return;
             }
         }
         // Call again open to try the next server.
-        io_service_->post(std::bind(&Exchange::openNext,
+        io_service_->post(std::bind(&UdpExchange::openNext,
                                     shared_from_this()));
         return;
     }
@@ -453,21 +504,22 @@ Exchange::open() {
         buffer_ = sent_->getBuffer();
         size_ = buffer_.size();
 
-        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_EXCHANGE_SEND_RETRY)
+        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                  RADIUS_UDP_EXCHANGE_SEND_RETRY)
             .arg(identifier_)
             .arg(buffer_.size())
             .arg(retries_);
 
         socket_->asyncSend(&buffer_[0],
-                                buffer_.size(),
-                                ep_.get(),
-                                std::bind(&Exchange::sentHandler,
-                                          shared_from_this(),
-                                          ph::_1,   // error_code.
-                                          ph::_2)); // size.
+                           buffer_.size(),
+                           ep_.get(),
+                           std::bind(&UdpExchange::sentHandler,
+                                     shared_from_this(),
+                                     ph::_1,   // error_code.
+                                     ph::_2)); // size.
         return;
     } catch (const Exception& exc) {
-        LOG_ERROR(radius_logger, RADIUS_EXCHANGE_OPEN_FAILED)
+        LOG_ERROR(radius_logger, RADIUS_UDP_EXCHANGE_OPEN_FAILED)
             .arg(identifier_)
             .arg(exc.what());
         cancelTimer();
@@ -476,16 +528,16 @@ Exchange::open() {
             socket_->close();
             socket_.reset();
         }
-        io_service_->post(std::bind(&Exchange::openNext,
+        io_service_->post(std::bind(&UdpExchange::openNext,
                                     shared_from_this()));
         return;
     }
 }
 
 void
-Exchange::sentHandler(ExchangePtr ex,
-                      const boost::system::error_code ec,
-                      const size_t size) {
+UdpExchange::sentHandler(UdpExchangePtr ex,
+                         const boost::system::error_code ec,
+                         const size_t size) {
     if (!ex) {
         isc_throw(Unexpected, "null exchange in sentHandler");
     }
@@ -503,7 +555,7 @@ Exchange::sentHandler(ExchangePtr ex,
 
     // Check error code.
     if (ec) {
-        LOG_ERROR(radius_logger, RADIUS_EXCHANGE_SEND_FAILED)
+        LOG_ERROR(radius_logger, RADIUS_UDP_EXCHANGE_SEND_FAILED)
             .arg(ex->identifier_)
             .arg(ec.message());
         ex->cancelTimer();
@@ -511,27 +563,116 @@ Exchange::sentHandler(ExchangePtr ex,
             ex->socket_->close();
             ex->socket_.reset();
         }
-        ex->io_service_->post(std::bind(&Exchange::openNext, ex));
+        ex->io_service_->post(std::bind(&UdpExchange::openNext, ex));
         return;
     }
 
     // No error: receive response.
-    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_EXCHANGE_SENT)
+    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_UDP_EXCHANGE_SENT)
         .arg(ex->identifier_)
         .arg(size);
     ex->buffer_.clear();
     ex->buffer_.resize(BUF_LEN);
     ex->size_ = ex->buffer_.size();
     ex->socket_->asyncReceive(&(ex->buffer_)[0], ex->size_, 0, ex->ep_.get(),
-                              std::bind(&Exchange::receivedHandler, ex,
+                              std::bind(&UdpExchange::receivedHandler,
+                                        ex,
                                         ph::_1,   // error_code.
                                         ph::_2)); // size.
 }
 
 void
-Exchange::receivedHandler(ExchangePtr ex,
-                          const boost::system::error_code ec,
-                          const size_t size) {
+Exchange::processResponse() {
+    // Decode message.
+    rc_ = OK_RC;
+    try {
+        // In order:
+        //  - decode message.
+        //  - verify that it is signed.
+        //  - verify that identifiers match.
+        //  - verify that message codes match.
+        received_->decode();
+        if (RadiusImpl::instance().use_message_authenticator_) {
+            auto attrs = received_->getAttributes();
+            if (!attrs || (attrs->count(PW_MESSAGE_AUTHENTICATOR) == 0)) {
+                isc_throw(BadValue, "missing Message-Authenticator");
+            }
+        }
+        unsigned got = received_->getIdentifier();
+        unsigned expected = sent_->getIdentifier();
+        if (got != expected) {
+            LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_MISMATCH)
+                .arg(identifier_)
+                .arg(got)
+                .arg(expected);
+            rc_ = BADRESP_RC;
+        } else if (request_->getCode() == PW_ACCESS_REQUEST) {
+            if (received_->getCode() == PW_ACCESS_REJECT) {
+                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                          RADIUS_EXCHANGE_RECEIVED_ACCESS_REJECT)
+                    .arg(identifier_);
+                rc_ = REJECT_RC;
+            } else if (received_->getCode() != PW_ACCESS_ACCEPT) {
+                LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_UNEXPECTED)
+                    .arg(identifier_)
+                    .arg(msgCodeToText(request_->getCode()))
+                    .arg(msgCodeToText(received_->getCode()));
+                rc_ = BADRESP_RC;
+            } else {
+                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                          RADIUS_EXCHANGE_RECEIVED_ACCESS_ACCEPT)
+                    .arg(identifier_);
+            }
+        } else if (request_->getCode() == PW_ACCOUNTING_REQUEST) {
+            if (received_->getCode() != PW_ACCOUNTING_RESPONSE) {
+                LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_UNEXPECTED)
+                    .arg(identifier_)
+                    .arg(msgCodeToText(request_->getCode()))
+                    .arg(msgCodeToText(received_->getCode()));
+                rc_ = BADRESP_RC;
+            } else {
+                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                          RADIUS_EXCHANGE_RECEIVED_ACCOUNTING_RESPONSE)
+                    .arg(identifier_);
+            }
+        } else if (request_->getCode() == PW_STATUS_SERVER) {
+            if (received_->getCode() == PW_ACCESS_ACCEPT) {
+                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                          RADIUS_EXCHANGE_RECEIVED_ACCESS_ACCEPT)
+                    .arg(identifier_);
+            } else if (received_->getCode() == PW_ACCESS_REJECT) {
+                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                          RADIUS_EXCHANGE_RECEIVED_ACCESS_REJECT)
+                    .arg(identifier_);
+            } else if (received_->getCode() == PW_ACCOUNTING_RESPONSE) {
+                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                          RADIUS_EXCHANGE_RECEIVED_ACCOUNTING_RESPONSE)
+                    .arg(identifier_);
+            } else {
+                LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_UNEXPECTED)
+                    .arg(identifier_)
+                    .arg(msgCodeToText(request_->getCode()))
+                    .arg(msgCodeToText(received_->getCode()));
+                rc_ = BADRESP_RC;
+            }
+        }
+    } catch (const Exception& exc) {
+        LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_BAD_RESPONSE)
+            .arg(identifier_)
+            .arg(exc.what());
+        rc_ = BADRESP_RC;
+    }
+
+    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+              RADIUS_EXCHANGE_RECEIVED_RESPONSE)
+        .arg(identifier_)
+        .arg(exchangeRCtoText(rc_));
+}
+
+void
+UdpExchange::receivedHandler(UdpExchangePtr ex,
+                             const boost::system::error_code ec,
+                             const size_t size) {
     if (!ex) {
         isc_throw(Unexpected, "null exchange in receivedHandler");
     }
@@ -556,10 +697,10 @@ Exchange::receivedHandler(ExchangePtr ex,
 
     // Check error code.
     if (ec) {
-        LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVE_FAILED)
+        LOG_ERROR(radius_logger, RADIUS_UDP_EXCHANGE_RECEIVE_FAILED)
             .arg(ex->identifier_)
             .arg(ec.message());
-        ex->io_service_->post(std::bind(&Exchange::openNext, ex));
+        ex->io_service_->post(std::bind(&UdpExchange::openNext, ex));
         return;
     }
 
@@ -571,102 +712,26 @@ Exchange::receivedHandler(ExchangePtr ex,
     }
 
     // Create message.
-    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_EXCHANGE_RECEIVED)
+    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_UDP_EXCHANGE_RECEIVED)
         .arg(ex->identifier_)
         .arg(size);
     ex->buffer_.resize(size);
     ex->received_.reset(new Message(ex->buffer_, ex->sent_->getAuth(),
                                     ex->server_->getSecret()));
 
-    // Decode message.
-    ex->rc_ = OK_RC;
-    try {
-        // In order:
-        //  - decode message.
-        //  - verify that identifiers match.
-        //  - verify that message codes match.
-        ex->received_->decode();
-        unsigned got = ex->received_->getIdentifier();
-        unsigned expected = ex->sent_->getIdentifier();
-        if (got != expected) {
-            LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_MISMATCH)
-                .arg(ex->identifier_)
-                .arg(got)
-                .arg(expected);
-            ex->rc_ = BADRESP_RC;
-        } else if (ex->request_->getCode() == PW_ACCESS_REQUEST) {
-            if (ex->received_->getCode() == PW_ACCESS_REJECT) {
-                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-                          RADIUS_EXCHANGE_RECEIVED_ACCESS_REJECT)
-                    .arg(ex->identifier_);
-                ex->rc_ = REJECT_RC;
-            } else if (ex->received_->getCode() != PW_ACCESS_ACCEPT) {
-                LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_UNEXPECTED)
-                    .arg(ex->identifier_)
-                    .arg(msgCodeToText(ex->request_->getCode()))
-                    .arg(msgCodeToText(ex->received_->getCode()));
-                ex->rc_ = BADRESP_RC;
-            } else {
-                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-                          RADIUS_EXCHANGE_RECEIVED_ACCESS_ACCEPT)
-                    .arg(ex->identifier_);
-            }
-        } else if (ex->request_->getCode() == PW_ACCOUNTING_REQUEST) {
-            if (ex->received_->getCode() != PW_ACCOUNTING_RESPONSE) {
-                LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_UNEXPECTED)
-                    .arg(ex->identifier_)
-                    .arg(msgCodeToText(ex->request_->getCode()))
-                    .arg(msgCodeToText(ex->received_->getCode()));
-                ex->rc_ = BADRESP_RC;
-            } else {
-                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-                          RADIUS_EXCHANGE_RECEIVED_ACCOUNTING_RESPONSE)
-                    .arg(ex->identifier_);
-            }
-        } else if (ex->request_->getCode() == PW_STATUS_SERVER) {
-            if (ex->received_->getCode() == PW_ACCESS_ACCEPT) {
-                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-                          RADIUS_EXCHANGE_RECEIVED_ACCESS_ACCEPT)
-                    .arg(ex->identifier_);
-            } else if (ex->received_->getCode() == PW_ACCESS_REJECT) {
-                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-                          RADIUS_EXCHANGE_RECEIVED_ACCESS_REJECT)
-                    .arg(ex->identifier_);
-            } else if (ex->received_->getCode() == PW_ACCOUNTING_RESPONSE) {
-                LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-                          RADIUS_EXCHANGE_RECEIVED_ACCOUNTING_RESPONSE)
-                    .arg(ex->identifier_);
-            } else {
-                LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_UNEXPECTED)
-                    .arg(ex->identifier_)
-                    .arg(msgCodeToText(ex->request_->getCode()))
-                    .arg(msgCodeToText(ex->received_->getCode()));
-                ex->rc_ = BADRESP_RC;
-            }
-        }
-    } catch (const Exception& exc) {
-        LOG_ERROR(radius_logger, RADIUS_EXCHANGE_RECEIVED_BAD_RESPONSE)
-            .arg(ex->identifier_)
-            .arg(exc.what());
-        ex->rc_ = BADRESP_RC;
-    }
-
-    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
-              RADIUS_EXCHANGE_RECEIVED_RESPONSE)
-        .arg(ex->identifier_)
-        .arg(exchangeRCtoText(ex->rc_));
+    ex->processResponse();
 
     // If bad then retry, if not including reject it is done.
     if ((ex->rc_ != OK_RC) && (ex->rc_ != REJECT_RC)) {
-        ex->io_service_->post(std::bind(&Exchange::openNext, ex));
+        ex->io_service_->post(std::bind(&UdpExchange::openNext, ex));
     } else {
         ex->logReplyMessages();
-        ex->io_service_->post(std::bind(&Exchange::terminate, ex));
+        ex->io_service_->post(std::bind(&UdpExchange::terminate, ex));
     }
 }
 
 void
-Exchange::terminate() {
+UdpExchange::terminate() {
     // Avoid multiple terminations.
     MultiThreadingLock lock(*mutex_);
 
@@ -684,11 +749,12 @@ Exchange::terminate() {
     }
 
     if ((rc_ != OK_RC) && (rc_ != REJECT_RC)) {
-        LOG_ERROR(radius_logger, RADIUS_EXCHANGE_FAILED)
+        LOG_ERROR(radius_logger, RADIUS_UDP_EXCHANGE_FAILED)
             .arg(identifier_)
             .arg(exchangeRCtoText(rc_));
     } else {
-        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_EXCHANGE_TERMINATE)
+        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE,
+                  RADIUS_UDP_EXCHANGE_TERMINATE)
             .arg(identifier_)
             .arg(exchangeRCtoText(rc_));
     }
@@ -716,15 +782,15 @@ Exchange::terminate() {
 }
 
 void
-Exchange::setTimer() {
+UdpExchange::setTimer() {
     cancelTimer();
     timer_.reset(new IntervalTimer(io_service_));
-    timer_->setup(std::bind(&Exchange::timeoutHandler, shared_from_this()),
+    timer_->setup(std::bind(&UdpExchange::timeoutHandler, shared_from_this()),
                   server_->getTimeout() * 1000, IntervalTimer::ONE_SHOT);
 }
 
 void
-Exchange::cancelTimer() {
+UdpExchange::cancelTimer() {
     if (timer_) {
         timer_->cancel();
         timer_.reset();
@@ -732,9 +798,9 @@ Exchange::cancelTimer() {
 }
 
 void
-Exchange::timeoutHandler(ExchangePtr ex) {
+UdpExchange::timeoutHandler(UdpExchangePtr ex) {
     MultiThreadingLock lock(*ex->mutex_);
-    LOG_ERROR(radius_logger, RADIUS_EXCHANGE_TIMEOUT)
+    LOG_ERROR(radius_logger, RADIUS_UDP_EXCHANGE_TIMEOUT)
         .arg(ex->identifier_);
     ex->rc_ = TIMEOUT_RC;
     ex->cancelTimer();
@@ -743,5 +809,173 @@ Exchange::timeoutHandler(ExchangePtr ex) {
     }
 }
 
+TcpExchange::TcpExchange(const MessagePtr& request,
+                         unsigned maxretries,
+                         const Servers& servers,
+                         Handler handler)
+    : Exchange(request, maxretries, servers, handler),
+      start_time_(std::chrono::steady_clock().now()),
+      server_(), response_() {
+    server_ = servers_[0];
+}
+
+void
+TcpExchange::start() {
+    if (RadiusImpl::shutdown_) {
+        shutdown();
+    }
+
+    if (!server_) {
+        isc_throw(Unexpected, "no server");
+    }
+
+    if (!RadiusImpl::instance().tcp_client_) {
+        isc_throw(Unexpected, "no TCP client");
+    }
+
+    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_TCP_EXCHANGE_START)
+        .arg(identifier_);
+
+    try {
+
+        // Reset error code.
+        rc_ = ERROR_RC;
+
+        // Build to be send request message.
+        buildRequest();
+
+        // Build write data request.
+        WireDataPtr request(new WireData(sent_->getBuffer()));
+
+        // Build write data response.
+        response_.reset(new WireData());
+
+        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_TCP_EXCHANGE_SEND)
+            .arg(identifier_)
+            .arg(request->size())
+            .arg(server_->getPeerAddress().toText())
+            .arg(server_->getPeerPort())
+            .arg(server_->getTlsContext() ? " using TLS" : "");
+
+        RadiusImpl::instance().tcp_client_->asyncSendRequest(
+            server_->getPeerAddress(),
+            server_->getPeerPort(),
+            server_->getTlsContext(),
+            request,
+            response_,
+            true,
+            TcpExchange::CompleteCheck,
+            std::bind(&TcpExchange::RequestHandler,
+                      shared_from_this(),
+                      ph::_1,   // error_code
+                      ph::_2,   // response
+                      ph::_3),  // error_msg
+            TcpClient::RequestTimeout(server_->getTimeout() * 1000));
+    } catch (const Exception& exc) {
+        LOG_ERROR(radius_logger, RADIUS_TCP_EXCHANGE_START_ERROR)
+            .arg(identifier_)
+            .arg(exc.what());
+        rc_ = ERROR_RC;
+        // Call handler.
+        if (handler_) {
+            auto handler = handler_;
+            // Avoid to keep a circular reference.
+            handler_ = Handler();
+            handler(shared_from_this());
+        }
+    }
+}
+
+void
+TcpExchange::shutdown() {
+    handler_ = Handler();
+}
+
+void
+TcpExchange::RequestHandler(TcpExchangePtr ex,
+                            const boost::system::error_code& ec,
+                            const WireDataPtr& response,
+                            const string& error_msg) {
+    if (!ex) {
+        isc_throw(Unexpected, "null exchange in RequestHandler");
+    }
+
+    if (RadiusImpl::shutdown_) {
+        return;
+    }
+
+    // Call handler.
+    auto call_handler = [](TcpExchangePtr exchange) {
+        if (exchange->handler_) {
+            auto handler = exchange->handler_;
+            // Avoid to keep a circular reference.
+            exchange->handler_ = Handler();
+            handler(exchange);
+        }
+    };
+
+    // Check error code.
+    if (ec) {
+        LOG_ERROR(radius_logger, RADIUS_TCP_EXCHANGE_RECEIVE_FAILED)
+            .arg(ex->identifier_)
+            .arg(error_msg);
+        if (ec == boost::asio::error::timed_out) {
+            ex->rc_ = TIMEOUT_RC;
+        } else {
+            ex->rc_ = ERROR_RC;
+        }
+        call_handler(ex);
+        return;
+    }
+
+    if (!response) {
+        isc_throw(Unexpected, "null response in RequestHandler");
+    }
+
+    LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_TCP_EXCHANGE_RECEIVED)
+        .arg(ex->identifier_)
+        .arg(response->size());
+
+    const WireData& buffer = *response;
+    ex->received_.reset(new Message(buffer, ex->sent_->getAuth(),
+                                    ex->server_->getSecret()));
+
+    ex->processResponse();
+
+    if ((ex->rc_ == OK_RC) || (ex->rc_ == REJECT_RC)) {
+        ex->logReplyMessages();
+        LOG_DEBUG(radius_logger, RADIUS_DBG_TRACE, RADIUS_TCP_EXCHANGE_SUCCESS)
+            .arg(ex->identifier_)
+            .arg(exchangeRCtoText(ex->rc_));
+    } else {
+        LOG_ERROR(radius_logger, RADIUS_TCP_EXCHANGE_FAILURE)
+            .arg(ex->identifier_)
+            .arg(exchangeRCtoText(ex->rc_));
+    }
+    call_handler(ex);
+}
+
+int
+TcpExchange::CompleteCheck(const WireDataPtr& response, string& error_msg) {
+    if (!response) {
+        error_msg = "null response";
+        return (-1);
+    }
+    const WireData& buffer = *response;
+    if (buffer.size() < AUTH_HDR_LEN) {
+        return (0);
+    }
+    uint16_t length = static_cast<uint16_t>(buffer[2]) << 8;
+    length |= static_cast<uint16_t>(buffer[3]);
+    if (length > buffer.size()) {
+        return (0);
+    } else if (length == buffer.size()) {
+        return (1);
+    } else {
+        error_msg = "overflow";
+        return (-2);
+    }
+}
+
 } // end of namespace isc::radius
 } // end of namespace isc
