@@ -17,6 +17,7 @@
 #include <radius_accounting.h>
 #include <radius_log.h>
 #include <radius_parsers.h>
+#include <radius_tls.h>
 
 #include <exception>
 
@@ -26,11 +27,136 @@ using namespace isc::asiolink;
 using namespace isc::data;
 using namespace isc::db;
 using namespace isc::dhcp;
+using namespace isc::tcp;
 using namespace isc::util;
 
 namespace isc {
 namespace radius {
 
+UdpClient::UdpClient(const IOServicePtr& io_service, unsigned thread_pool_size)
+    : io_service_(io_service), thread_pool_size_(thread_pool_size) {
+    // Do nothing in ST mode.
+    if (thread_pool_size == 0) {
+        return;
+    }
+
+    // Create our own private IOService.
+    thread_io_service_.reset(new IOService());
+    thread_pool_ =
+        boost::make_shared<IoServiceThreadPool>(thread_io_service_,
+                                                thread_pool_size_);
+
+    // Add critical section callbacks.
+    MultiThreadingMgr::instance().addCriticalSectionCallbacks("RADIUS",
+        [this]() { checkPermissions(); },
+        [this]() { pause(); },
+        [this]() { resume(); });
+}
+
+UdpClient::~UdpClient() {
+    stop();
+}
+
+unsigned
+UdpClient::getThreadPoolSize() const {
+    return (thread_pool_size_);
+}
+
+const asiolink::IOServicePtr
+UdpClient::getThreadIOService() const {
+    return (thread_io_service_);
+}
+
+void
+UdpClient::start() {
+    if (thread_pool_) {
+        thread_pool_->run();
+
+        LOG_INFO(radius_logger, RADIUS_THREAD_POOL_STARTED)
+            .arg(thread_pool_size_);
+    }
+}
+
+void
+UdpClient::stop() {
+    if (thread_pool_) {
+        MultiThreadingMgr::instance().removeCriticalSectionCallbacks("RADIUS");
+        thread_pool_->stop();
+        for (auto const& exchange : exchange_list_) {
+            exchange->shutdown();
+        }
+        thread_pool_->getIOService()->stopAndPoll();
+        thread_pool_.reset();
+    } else {
+        for (auto const& exchange : exchange_list_) {
+            exchange->shutdown();
+        }
+        io_service_->stopAndPoll();
+    }
+    exchange_list_.clear();
+}
+
+void
+UdpClient::checkPermissions() {
+    // Since this function is used as CS callback all exceptions must be
+    // suppressed, unlikely though they may be.
+    try {
+        if (thread_pool_) {
+            thread_pool_->checkPausePermissions();
+        }
+    } catch (const isc::MultiThreadingInvalidOperation& ex) {
+        LOG_ERROR(radius_logger, RADIUS_PAUSE_ILLEGAL)
+            .arg(ex.what());
+        // The exception needs to be propagated to the caller of the
+        // MultiThreadingCriticalSection constructor.
+        throw;
+    } catch (const exception& ex) {
+        LOG_ERROR(radius_logger, RADIUS_PAUSE_PERMISSIONS_FAILED)
+            .arg(ex.what());
+    }
+}
+
+void
+UdpClient::pause() {
+    // Since this function is used as CS callback all exceptions must be
+    // suppressed, unlikely though they may be.
+    try {
+        // Pause the thread pool.
+        if (thread_pool_) {
+            thread_pool_->pause();
+        }
+    } catch (const exception& ex) {
+        LOG_ERROR(radius_logger, RADIUS_PAUSE_FAILED)
+            .arg(ex.what());
+    }
+}
+
+void
+UdpClient::resume() {
+    // Since this function is used as CS callback all exceptions must be
+    // suppressed, unlikely though they may be.
+    try {
+        if (thread_pool_) {
+            thread_pool_->run();
+        }
+    } catch (const exception& ex) {
+        LOG_ERROR(radius_logger, RADIUS_RESUME_FAILED)
+            .arg(ex.what());
+    }
+}
+
+void
+UdpClient::registerExchange(ExchangePtr exchange) {
+    MultiThreadingLock lock(mutex_);
+    exchange_list_.push_back(exchange);
+}
+
+void
+UdpClient::unregisterExchange(ExchangePtr exchange) {
+    MultiThreadingLock lock(mutex_);
+    exchange_list_.remove(exchange);
+}
+
 std::atomic<bool> RadiusImpl::shutdown_(false);
 
 RadiusImpl&
@@ -45,13 +171,16 @@ RadiusImpl::instancePtr() {
 }
 
 RadiusImpl::RadiusImpl()
-    : auth_(new RadiusAccess()), acct_(new RadiusAccounting()),
+    : proto_(PW_PROTO_UDP), udp_client_(), tcp_client_(),
+      tls_(new RadiusTls()),
+      auth_(new RadiusAccess()), acct_(new RadiusAccounting()),
       bindaddr_("*"), canonical_mac_address_(false),
       clientid_pop0_(false), clientid_printable_(false),
       deadtime_(0), extract_duid_(true),
       reselect_subnet_pool_(false), reselect_subnet_address_(false),
       retries_(3), thread_pool_size_(0), timeout_(0),
       id_type4_(Host::IDENT_CLIENT_ID), id_type6_(Host::IDENT_DUID),
+      use_message_authenticator_(false),
       io_context_(new IOService()), io_service_(io_context_) {
 }
 
@@ -65,13 +194,15 @@ RadiusImpl::~RadiusImpl() {
 }
 
 void RadiusImpl::registerExchange(ExchangePtr exchange) {
-    MultiThreadingLock lock(mutex_);
-    exchange_list_.push_back(exchange);
+    if (udp_client_) {
+        udp_client_->registerExchange(exchange);
+    }
 }
 
 void RadiusImpl::unregisterExchange(ExchangePtr exchange) {
-    MultiThreadingLock lock(mutex_);
-    exchange_list_.remove(exchange);
+    if (udp_client_) {
+        udp_client_->unregisterExchange(exchange);
+    }
 }
 
 void RadiusImpl::cleanup() {
@@ -88,7 +219,6 @@ void RadiusImpl::cleanup() {
     canonical_mac_address_ = false;
     cache_.reset();
     bindaddr_ = "*";
-    thread_pool_size_ = 0;
     remap_.clear();
 
     if (backend_) {
@@ -97,29 +227,36 @@ void RadiusImpl::cleanup() {
         backend_.reset();
     }
 
+    if (udp_client_) {
+        udp_client_->stop();
+    }
+
+    if (tcp_client_) {
+        tcp_client_->stop();
+    }
+
+    tls_.reset(new RadiusTls());
     auth_.reset(new RadiusAccess());
     acct_.reset(new RadiusAccounting());
 
-    if (thread_pool_) {
-        MultiThreadingMgr::instance().removeCriticalSectionCallbacks("RADIUS");
-        thread_pool_->stop();
-        for (auto const& exchange : exchange_list_) {
-            exchange->shutdown();
-        }
-        thread_pool_->getIOService()->stopAndPoll();
-        thread_pool_.reset();
-    } else {
-        for (auto const& exchange : exchange_list_) {
-            exchange->shutdown();
-        }
-        io_context_->stopAndPoll();
+    if (udp_client_) {
+        udp_client_.reset();
+    }
+
+    if (tcp_client_) {
+        tcp_client_.reset();
+    }
+
+    if (getIOContext()) {
+        getIOContext()->stopAndPoll();
     }
 
     io_context_.reset(new IOService());
-    exchange_list_.clear();
+
     if (getIOService()) {
         getIOService()->stopAndPoll();
     }
+
     io_service_ = io_context_;
 }
 
@@ -130,6 +267,7 @@ void RadiusImpl::reset() {
 }
 
 void RadiusImpl::init(ElementPtr&config) {
+    tls_.reset(new RadiusTls());
     auth_.reset(new RadiusAccess());
     acct_.reset(new RadiusAccounting());
     RadiusConfigParser parser;
@@ -159,6 +297,7 @@ RadiusImpl::startServices() {
     ConstElementPtr const& dhcp_config(
         CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading());
     bool multi_threaded(false);
+    unsigned thread_pool_size(0);
     uint32_t dhcp_threads(0);
     uint32_t dummy_queue_size(0);
     CfgMultiThreading::extract(dhcp_config, multi_threaded, dhcp_threads,
@@ -167,92 +306,125 @@ RadiusImpl::startServices() {
     if (multi_threaded) {
         // When threads are configured as zero, use the same number as DHCP
         // threads. If that is also zero, auto-detect.
-        unsigned thread_pool_size(thread_pool_size_);
         if (thread_pool_size_ == 0) {
             if (dhcp_threads == 0) {
                 uint32_t const hardware_threads(
                     MultiThreadingMgr::detectThreadCount());
                 if (hardware_threads == 0) {
                     // Keep it single-threaded.
-                    return;
+                    multi_threaded = false;
+                    thread_pool_size = 0;
                 } else {
                     thread_pool_size = hardware_threads;
                 }
             } else {
                 thread_pool_size = dhcp_threads;
             }
+        } else {
+            thread_pool_size = thread_pool_size_;
         }
+    }
 
+    if (multi_threaded) {
         // Schedule a start of the services. This ensures we begin after
         // the dust has settled and Kea MT mode has been firmly established.
-        io_service_->post([this, thread_pool_size]() {
-            // Initialize thread pool.
-            thread_pool_ =
-                boost::make_shared<IoServiceThreadPool>(IOServicePtr(),
-                                                        thread_pool_size);
-            io_context_ = thread_pool_->getIOService();
+        if (proto_ == PW_PROTO_UDP) {
+            io_service_->post([this, thread_pool_size]() {
+                udp_client_.reset(new UdpClient(io_service_,
+                                                thread_pool_size));
 
-            // Add critical section callbacks.
-            MultiThreadingMgr::instance().addCriticalSectionCallbacks("RADIUS",
-                    [this]() { checkPausePermissions(); },
-                    [this]() { pauseThreadPool(); },
-                    [this]() { resumeThreadPool(); });
+                io_context_ = udp_client_->getThreadIOService();
 
-            // Run the thread pool.
-            thread_pool_->run();
+                udp_client_->start();
+            });
+        } else {
+            io_service_->post([this, multi_threaded, thread_pool_size]() {
+                tcp_client_.reset(new TcpClient(io_service_,
+                                                multi_threaded,
+                                                thread_pool_size,
+                                                true));
 
-            LOG_INFO(radius_logger, RADIUS_THREAD_POOL_STARTED)
-                .arg(thread_pool_size);
-        });
+                io_context_ = tcp_client_->getThreadIOService();
+
+                tcp_client_->start();
+            });
+        }
+    } else {
+        if (proto_ == PW_PROTO_UDP) {
+            udp_client_.reset(new UdpClient(io_service_, 0));
+        } else {
+            tcp_client_.reset(new TcpClient(io_service_, false, 0));
+        }
+    }
+}
+
+bool
+RadiusImpl::serveAccess() const {
+    if (shutdown_) {
+        return (false);
+    }
+    if (!auth_ || !auth_->enabled_) {
+        return (false);
+    }
+    if (proto_ != PW_PROTO_TLS) {
+        return (true);
+    }
+    return (tls_ && tls_->enabled_);
+}
+
+bool
+RadiusImpl::serveAccounting() const {
+    if (shutdown_) {
+        return (false);
+    }
+    if (!acct_ || !acct_->enabled_) {
+        return (false);
+    }
+    if (proto_ != PW_PROTO_TLS) {
+        return (true);
+    }
+    return (tls_ && tls_->enabled_);
+}
+
+const Servers&
+RadiusImpl::getAccessServers() const {
+    if (proto_ != PW_PROTO_TLS) {
+        return (auth_->servers_);
+    } else {
+        return (tls_->servers_);
+    }
+}
+
+const Servers&
+RadiusImpl::getAccountingServers() const {
+    if (proto_ != PW_PROTO_TLS) {
+        return (acct_->servers_);
+    } else {
+        return (tls_->servers_);
     }
 }
 
 void
-RadiusImpl::checkPausePermissions() {
-    // Since this function is used as CS callback all exceptions must be
-    // suppressed, unlikely though they may be.
-    try {
-        if (thread_pool_) {
-            thread_pool_->checkPausePermissions();
-        }
-    } catch (const isc::MultiThreadingInvalidOperation& ex) {
-        LOG_ERROR(radius_logger, RADIUS_PAUSE_ILLEGAL)
-            .arg(ex.what());
-        // The exception needs to be propagated to the caller of the
-        // MultiThreadingCriticalSection constructor.
-        throw;
-    } catch (const exception& ex) {
-        LOG_ERROR(radius_logger, RADIUS_PAUSE_PERMISSIONS_FAILED)
-            .arg(ex.what());
+RadiusImpl::setAccessIdleTimer() {
+    if (shutdown_) {
+        return;
+    }
+    if (proto_ != PW_PROTO_TLS) {
+        auth_->setIdleTimer();
+    } else {
+        tls_->setIdleTimer();
     }
 }
 
 void
-RadiusImpl::pauseThreadPool() {
-    // Since this function is used as CS callback all exceptions must be
-    // suppressed, unlikely though they may be.
-    try {
-        // Pause the thread pool.
-        if (thread_pool_) {
-            thread_pool_->pause();
-        }
-    } catch (const exception& ex) {
-        LOG_ERROR(radius_logger, RADIUS_PAUSE_FAILED)
-            .arg(ex.what());
+RadiusImpl::setAccountingIdleTimer() {
+    if (shutdown_) {
+        return;
     }
-}
-
-void
-RadiusImpl::resumeThreadPool() {
-    // Since this function is used as CS callback all exceptions must be
-    // suppressed, unlikely though they may be.
-    try {
-        if (thread_pool_) {
-            thread_pool_->run();
-        }
-    } catch (const exception& ex) {
-        LOG_ERROR(radius_logger, RADIUS_RESUME_FAILED)
-            .arg(ex.what());
+    if (proto_ != PW_PROTO_TLS) {
+        acct_->setIdleTimer();
+    } else {
+        tls_->setIdleTimer();
     }
 }
 
@@ -416,6 +588,9 @@ ElementPtr RadiusImpl::toElement() const {
     // dictionary.
     result->set("dictionary", Element::create(dictionary_));
 
+    // protocol.
+    result->set("protocol", Element::create(protocolToText(proto_)));
+
     // bindaddr.
     result->set("bindaddr", Element::create(bindaddr_));
 
@@ -463,7 +638,14 @@ ElementPtr RadiusImpl::toElement() const {
     // timeout.
     result->set("timeout", Element::create(timeout_));
 
+    // use-message-authenticator.
+    result->set("use-message-authenticator",
+                Element::create(use_message_authenticator_));
+
     // services.
+    if (proto_ == PW_PROTO_TLS) {
+        result->set("tls", tls_->toElement());
+    }
     result->set("access", auth_->toElement());
     result->set("accounting", acct_->toElement());
 
