@@ -55,9 +55,6 @@ extern const isc::log::MessageID RADIUS_CONFIGURATION_FAILED;
 extern const isc::log::MessageID RADIUS_DECODE_MESSAGE;
 extern const isc::log::MessageID RADIUS_DEINIT_OK;
 extern const isc::log::MessageID RADIUS_ENCODE_MESSAGE;
-extern const isc::log::MessageID RADIUS_EXCHANGE_FAILED;
-extern const isc::log::MessageID RADIUS_EXCHANGE_OPEN_FAILED;
-extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED;
 extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED_ACCESS_ACCEPT;
 extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED_ACCESS_REJECT;
 extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED_ACCOUNTING_RESPONSE;
@@ -65,15 +62,6 @@ extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED_BAD_RESPONSE;
 extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED_MISMATCH;
 extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED_RESPONSE;
 extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVED_UNEXPECTED;
-extern const isc::log::MessageID RADIUS_EXCHANGE_RECEIVE_FAILED;
-extern const isc::log::MessageID RADIUS_EXCHANGE_SEND_FAILED;
-extern const isc::log::MessageID RADIUS_EXCHANGE_SEND_NEW;
-extern const isc::log::MessageID RADIUS_EXCHANGE_SEND_RETRY;
-extern const isc::log::MessageID RADIUS_EXCHANGE_SENT;
-extern const isc::log::MessageID RADIUS_EXCHANGE_START;
-extern const isc::log::MessageID RADIUS_EXCHANGE_SYNC_RETURN;
-extern const isc::log::MessageID RADIUS_EXCHANGE_TERMINATE;
-extern const isc::log::MessageID RADIUS_EXCHANGE_TIMEOUT;
 extern const isc::log::MessageID RADIUS_HOOK_FAILED;
 extern const isc::log::MessageID RADIUS_INIT_OK;
 extern const isc::log::MessageID RADIUS_INTEGER_ATTRIBUTE_FROM_BYTES_FAILED;
@@ -97,7 +85,30 @@ extern const isc::log::MessageID RADIUS_SESSION_HISTORY_OPENED;
 extern const isc::log::MessageID RADIUS_SESSION_HISTORY_OPEN_FAILED;
 extern const isc::log::MessageID RADIUS_SESSION_HISTORY_STORED;
 extern const isc::log::MessageID RADIUS_SESSION_HISTORY_STORE_FAILED;
+extern const isc::log::MessageID RADIUS_TCP_EXCHANGE_FAILURE;
+extern const isc::log::MessageID RADIUS_TCP_EXCHANGE_RECEIVED;
+extern const isc::log::MessageID RADIUS_TCP_EXCHANGE_RECEIVE_FAILED;
+extern const isc::log::MessageID RADIUS_TCP_EXCHANGE_SEND;
+extern const isc::log::MessageID RADIUS_TCP_EXCHANGE_START;
+extern const isc::log::MessageID RADIUS_TCP_EXCHANGE_START_ERROR;
+extern const isc::log::MessageID RADIUS_TCP_EXCHANGE_SUCCESS;
 extern const isc::log::MessageID RADIUS_THREAD_POOL_STARTED;
+extern const isc::log::MessageID RADIUS_TLS_STATUS;
+extern const isc::log::MessageID RADIUS_TLS_STATUS_ERROR;
+extern const isc::log::MessageID RADIUS_TLS_STATUS_FAILED;
+extern const isc::log::MessageID RADIUS_TLS_STATUS_SUCCEED;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_FAILED;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_OPEN_FAILED;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_RECEIVED;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_RECEIVE_FAILED;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_SEND_FAILED;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_SEND_NEW;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_SEND_RETRY;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_SENT;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_START;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_SYNC_RETURN;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_TERMINATE;
+extern const isc::log::MessageID RADIUS_UDP_EXCHANGE_TIMEOUT;
 
 } // namespace radius
 } // namespace isc
