@@ -97,11 +97,11 @@ TEST_F(ConfigTest, defaults) {
     EXPECT_NO_THROW(impl_.init(config));
     string expected = "{ "
         "\"access\": {"
-        "   \"attributes\": [ ],"
+        "   \"enabled\": false,"
         "   \"idle-timer-interval\": 0"
         "}, "
         "\"accounting\": {"
-        "   \"attributes\": [ ],"
+        "   \"enabled\": false,"
         "   \"idle-timer-interval\": 0"
         "}, "
         "\"bindaddr\": \"*\", "
@@ -113,12 +113,14 @@ TEST_F(ConfigTest, defaults) {
         "\"extract-duid\": true, "
         "\"identifier-type4\": \"client-id\", "
         "\"identifier-type6\": \"duid\", "
+        "\"protocol\": \"UDP\", "
         "\"reselect-subnet-address\": false, "
         "\"reselect-subnet-pool\": false, "
         "\"retries\": 3, "
         "\"session-history\": \"\", "
         "\"thread-pool-size\": 0, "
-        "\"timeout\": 10"
+        "\"timeout\": 10, "
+        "\"use-message-authenticator\": true"
         " }";
     runToElementTest<RadiusImpl>(expected, impl_);
 }
@@ -135,41 +137,50 @@ TEST_F(ConfigTest, global) {
     config->set("extract-duid", Element::create(false));
     config->set("identifier-type4", Element::create("hw-address"));
     config->set("identifier-type6", Element::create("flex-id"));
+    config->set("protocol", Element::create("TLS"));
     config->set("reselect-subnet-address", Element::create(true));
     config->set("reselect-subnet-pool", Element::create(true));
     config->set("retries", Element::create(5));
     config->set("session-history", Element::create("/tmp/foobar"));
     config->set("thread-pool-size", Element::create(8));
     config->set("timeout", Element::create(12));
+    config->set("use-message-authenticator", Element::create(false));
     EXPECT_NO_THROW(impl_.init(config));
     string expected = "{ "
         "\"access\": {"
-        "   \"attributes\": [ ],"
+        "   \"enabled\": false,"
         "   \"idle-timer-interval\": 0"
         "}, "
         "\"accounting\": {"
-        "   \"attributes\": [ ],"
+        "   \"enabled\": false,"
         "   \"idle-timer-interval\": 0"
         "}, "
         "\"bindaddr\": \"127.0.0.1\", "
         "\"canonical-mac-address\": true, "
         "\"client-id-pop0\": true, "
         "\"client-id-printable\": true, "
+        "\"tls\": {"
+        "   \"enabled\": false,"
+        "   \"idle-timer-interval\": 0"
+        "}, "
         "\"deadtime\": 10, "
         "\"dictionary\": \"/dev/null\", "
         "\"extract-duid\": false, "
         "\"identifier-type4\": \"hw-address\", "
         "\"identifier-type6\": \"flex-id\", "
+        "\"protocol\": \"TLS\", "
         "\"reselect-subnet-address\": true, "
         "\"reselect-subnet-pool\": true, "
         "\"retries\": 5, "
         "\"session-history\": \"/tmp/foobar\", "
         "\"thread-pool-size\": 8, "
-        "\"timeout\": 12"
+        "\"timeout\": 12, "
+        "\"use-message-authenticator\": false"
         " }";
     runToElementTest<RadiusImpl>(expected, impl_);
 
     // Check state.
+    EXPECT_EQ(PW_PROTO_TLS, impl_.proto_);
     EXPECT_EQ("127.0.0.1", impl_.bindaddr_);
     EXPECT_TRUE(impl_.canonical_mac_address_);
     EXPECT_TRUE(impl_.clientid_pop0_);
@@ -185,6 +196,7 @@ TEST_F(ConfigTest, global) {
     EXPECT_EQ("/tmp/foobar", impl_.session_history_filename_);
     EXPECT_EQ(8, impl_.thread_pool_size_);
     EXPECT_EQ(12, impl_.timeout_);
+    EXPECT_FALSE(impl_.use_message_authenticator_);
     EXPECT_TRUE(impl_.remap_.empty());
 }
 
@@ -371,6 +383,12 @@ TEST_F(ConfigTest, badGlobal) {
              << (numeric_limits<long>::max() / 1000) << "]";
     EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected.str());
 
+    impl_.reset();
+    config = Element::createMap();
+    config->set("use-message-authenticator", Element::create(1));
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError,
+                     "boolValue() called on non-Bool Element");
+
     impl_.reset();
     config = Element::createMap();
     config->set("access", Element::create(false));
@@ -629,11 +647,13 @@ TEST_F(ConfigTest, services) {
         "   \"type\": 1, "
         "   \"data\": \"foobar\" "
         "} ],"
+        "\"enabled\": false,"
         "\"idle-timer-interval\": 0"
         "}";
     runToElementTest<RadiusService>(expected, *impl_.auth_);
 
     // Needs a server to be enabled.
+    EXPECT_FALSE(impl_.tls_->enabled_);
     EXPECT_FALSE(impl_.auth_->enabled_);
     EXPECT_FALSE(impl_.acct_->enabled_);
     config = Element::createMap();
@@ -670,7 +690,7 @@ TEST_F(ConfigTest, services) {
     EXPECT_NO_THROW(impl_.init(config));
 
     expected = "{ "
-        "\"attributes\": [ ], "
+        "\"enabled\": true,"
         "\"idle-timer-interval\": 0,"
         "\"servers\": [ {"
         "   \"deadtime\": 0, "
@@ -729,7 +749,7 @@ TEST_F(ConfigTest, services) {
     EXPECT_NO_THROW(impl_.init(config));
 
     expected = "{ "
-        "\"attributes\": [ ], "
+        "\"enabled\": true,"
         "\"idle-timer-interval\": 0,"
         "\"servers\": [ {"
         "   \"deadtime\": 0, "
@@ -760,6 +780,168 @@ TEST_F(ConfigTest, services) {
     EXPECT_EQ(16450, srv->getPeerPort());
 }
 
+// Verify syntax of tls service.
+TEST_F(ConfigTest, tls) {
+    // TCP is not implemented.
+    ElementPtr config = Element::createMap();
+    config->set("protocol", Element::create(string("TCP")));
+    string expected = "protocol 'TCP' is not supported";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+    config->set("protocol", Element::create(string("UDP")));
+    EXPECT_NO_THROW(impl_.init(config));
+
+    // Services can be empty.
+    ElementPtr tls = Element::createMap();
+    ElementPtr access = Element::createMap();
+    ElementPtr accounting = Element::createMap();
+    config->set("tls", tls);
+    config->set("access", access);
+    config->set("accounting", accounting);
+    expected = "'tls' service can't be configured when protocol ";
+    expected += "is not 'TLS'";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+    config->set("protocol", Element::create(string("TLS")));
+    EXPECT_NO_THROW(impl_.init(config));
+    EXPECT_FALSE(impl_.tls_->enabled_);
+    EXPECT_TRUE(impl_.auth_->enabled_);
+    EXPECT_TRUE(impl_.acct_->enabled_);
+
+    // Can't set enabled in tls.
+    tls->set("enabled", Element::create(true));
+    expected = "can't set enabled in 'tls' (parsing tls)";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+
+    // Overwrite enabled.
+    tls = Element::createMap();
+    config->set("tls", tls);
+    accounting->set("enabled", Element::create(false));
+    EXPECT_NO_THROW(impl_.init(config));
+    EXPECT_FALSE(impl_.tls_->enabled_);
+    EXPECT_TRUE(impl_.auth_->enabled_);
+    EXPECT_FALSE(impl_.acct_->enabled_);
+
+    accounting = Element::createMap();
+    config->set("accounting", accounting);
+    access->set("enabled", Element::create(false));
+    EXPECT_NO_THROW(impl_.init(config));
+    EXPECT_FALSE(impl_.tls_->enabled_);
+    EXPECT_FALSE(impl_.auth_->enabled_);
+    EXPECT_TRUE(impl_.acct_->enabled_);
+
+    // Servers is forbidden outside tls.
+    tls = Element::createMap();
+    access = Element::createMap();
+    accounting = Element::createMap();
+    config->set("tls", tls);
+    config->set("access", access);
+    config->set("accounting", accounting);
+    access->set("servers", Element::createList());
+    expected = "can't have servers entry in 'access' with TLS ";
+    expected += "(parsing access)";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+    access = Element::createMap();
+    config->set("access", access);
+    accounting->set("servers", Element::createList());
+    expected = "can't have servers entry in 'accounting' with TLS ";
+    expected += "(parsing accounting)";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+
+    // Idle timer interval is forbidden outside tls.
+    tls = Element::createMap();
+    access = Element::createMap();
+    accounting = Element::createMap();
+    config->set("tls", tls);
+    config->set("access", access);
+    config->set("accounting", accounting);
+    access->set("idle-timer-interval", Element::create(60));
+    expected = "can't have idle-timer-interval entry in 'access' with TLS ";
+    expected += "(parsing access)";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+    access = Element::createMap();
+    config->set("access", access);
+    accounting->set("idle-timer-interval", Element::create(60));
+    expected = "can't have idle-timer-interval entry in 'accounting' with TLS ";
+    expected += "(parsing accounting)";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+
+    // Attribute is forbidden in tls.
+    tls = Element::createMap();
+    access = Element::createMap();
+    accounting = Element::createMap();
+    config->set("tls", tls);
+    config->set("access", access);
+    config->set("accounting", accounting);
+    tls->set("attributes", Element::createList());
+    expected = "can't define attributes in 'tls' (parsing tls)";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+
+    // Set idle timer interval in tls.
+    tls = Element::createMap();
+    config->set("tls", tls);
+    tls->set("idle-timer-interval", Element::create(60));
+    EXPECT_NO_THROW(impl_.init(config));
+    EXPECT_EQ(60, impl_.tls_->idle_timer_interval_);
+    EXPECT_EQ(0, impl_.auth_->idle_timer_interval_);
+    EXPECT_EQ(0, impl_.acct_->idle_timer_interval_);
+
+    // Servers for tls require TLS.
+    ElementPtr servers = Element::createList();
+    ElementPtr server = Element::createMap();
+    server->set("name", Element::create("127.0.0.1"));
+    // Secret is not required with TLS as it defaults to "radsec".
+    servers->add(server);
+    tls->set("servers", servers);
+    expected = "missing parameter 'trust-anchor' (:0:0) (parsing tls)";
+    EXPECT_THROW_MSG(impl_.init(config), ConfigError, expected);
+    string trust_anchor = TEST_CA_DIR;
+    server->set("trust-anchor", Element::create(trust_anchor));
+    string cert_file = TEST_CA_DIR "/kea-client.crt";
+    server->set("cert-file", Element::create(cert_file));
+    string key_file = TEST_CA_DIR "/kea-client.key";
+    server->set("key-file", Element::create(key_file));
+    EXPECT_NO_THROW(impl_.init(config));
+    expected = "{ "
+        "\"access\": {"
+        "   \"enabled\": true,"
+        "   \"idle-timer-interval\": 0"
+        "}, "
+        "\"accounting\": {"
+        "   \"enabled\": true,"
+        "   \"idle-timer-interval\": 0"
+        "}, "
+        "\"bindaddr\": \"*\", "
+        "\"canonical-mac-address\": false, "
+        "\"client-id-pop0\": false, "
+        "\"client-id-printable\": false, "
+        "\"tls\": {"
+        "   \"enabled\": true,"
+        "   \"idle-timer-interval\": 60, "
+        "   \"servers\": [ {"
+        "     \"deadtime\": 0, "
+        "     \"local-address\": \"127.0.0.1\", "
+        "     \"peer-address\": \"127.0.0.1\", "
+        "     \"peer-port\": 2083, "
+        "     \"secret\": \"radsec\", "
+        "     \"timeout\": 10, "
+        "     \"tls-context\": true "
+        "} ] }, "
+        "\"deadtime\": 0, "
+        "\"dictionary\": \"" + string(DICTIONARY) + "\", "
+        "\"extract-duid\": true, "
+        "\"identifier-type4\": \"client-id\", "
+        "\"identifier-type6\": \"duid\", "
+        "\"protocol\": \"TLS\", "
+        "\"reselect-subnet-address\": false, "
+        "\"reselect-subnet-pool\": false, "
+        "\"retries\": 3, "
+        "\"session-history\": \"\", "
+        "\"thread-pool-size\": 0, "
+        "\"timeout\": 10, "
+        "\"use-message-authenticator\": false"
+        " }";
+    runToElementTest<RadiusImpl>(expected, impl_);
+}
+
 // Verify parsing of attributes.
 TEST_F(ConfigTest, attribute) {
     // Need the dictionary.
