@@ -1,17 +1,15 @@
-// Copyright (C) 2023-2025 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2023-2026 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-/// @file This file contains tests which exercise the attribute,
-/// attribute collection and config attribute collection classes.
-
 #include <config.h>
 
 #include <client_exchange.h>
 #include <testutils/gtest_utils.h>
 #include <testutils/test_to_element.h>
+#include <radius.h>
 #include <attribute_test.h>
 #include <gtest/gtest.h>
 #include <sstream>
@@ -34,13 +32,9 @@ TEST(TestExchange, async) {
     MessagePtr msg;
     Servers servers;
 
-    // No IO service.
-    EXPECT_THROW_MSG(exchange.reset(new Exchange(io_service, msg, 0, servers, Exchange::Handler())),
-                     BadValue, "null IO service");
-
     // No message.
-    io_service.reset(new IOService());
-    EXPECT_THROW_MSG(exchange.reset(new Exchange(io_service, msg, 0, servers, Exchange::Handler())),
+    EXPECT_THROW_MSG(Exchange::create(io_service, msg, 0, servers,
+                                      Exchange::Handler()),
                      BadValue, "null request");
 
     // No servers.
@@ -49,21 +43,31 @@ TEST(TestExchange, async) {
     AttributesPtr attrs;
     string secret = "foobar";
     ASSERT_NO_THROW_LOG(msg.reset(new Message(code, 0, auth, secret, attrs)));
-    EXPECT_THROW_MSG(exchange.reset(new Exchange(io_service, msg, 0, servers, Exchange::Handler())),
+    EXPECT_THROW_MSG(Exchange::create(io_service, msg, 0, servers,
+                                      Exchange::Handler()),
                      BadValue, "no server");
 
     // No handler.
     ServerPtr server;
     IOAddress addr("127.0.0.1");
-    ASSERT_NO_THROW_LOG(server.reset(new Server(addr, 11646, addr, secret, 0)));
+    TlsContextPtr tls_context;
+    ASSERT_NO_THROW_LOG(server.reset(new Server(addr, 11646, addr, tls_context,
+                                                secret, 0)));
+    ASSERT_TRUE(server);
     servers.push_back(server);
-    EXPECT_THROW_MSG(exchange.reset(new Exchange(io_service, msg, 0, servers, Exchange::Handler())),
+    EXPECT_THROW_MSG(Exchange::create(io_service, msg, 0, servers,
+                                      Exchange::Handler()),
                      BadValue, "null handler");
 
-    // No error.
+    // No IO service.
     auto handler = [] (const ExchangePtr) { };
-    ASSERT_NO_THROW_LOG(exchange.reset(new Exchange(io_service, msg, 0,
-                                                    servers, handler)));
+    EXPECT_THROW_MSG(Exchange::create(io_service, msg, 0, servers, handler),
+                     BadValue, "null IO service");
+
+    // No error.
+    io_service.reset(new IOService());
+    ASSERT_NO_THROW_LOG(exchange = Exchange::create(io_service, msg, 0,
+                                                    servers, handler));
 
     // Check exchange.
     ASSERT_TRUE(exchange);
@@ -75,6 +79,9 @@ TEST(TestExchange, async) {
     MessagePtr response = exchange->getResponse();
     EXPECT_FALSE(response);
 
+    // Verify it is an UDP Exchange.
+    EXPECT_TRUE(boost::dynamic_pointer_cast<UdpExchange>(exchange));
+
     // As a best practice, call any remaining handlers.
     io_service->stopAndPoll();
 }
@@ -86,7 +93,7 @@ TEST(TestExchange, sync) {
     Servers servers;
 
     // No message.
-    EXPECT_THROW_MSG(exchange.reset(new Exchange(msg, 0, servers)), BadValue,
+    EXPECT_THROW_MSG(Exchange::create(msg, 0, servers), BadValue,
                      "null request");
 
     // No servers.
@@ -95,15 +102,18 @@ TEST(TestExchange, sync) {
     AttributesPtr attrs;
     string secret = "foobar";
     ASSERT_NO_THROW_LOG(msg.reset(new Message(code, 0, auth, secret, attrs)));
-    EXPECT_THROW_MSG(exchange.reset(new Exchange(msg, 0, servers)), BadValue,
+    EXPECT_THROW_MSG(Exchange::create(msg, 0, servers), BadValue,
                      "no server");
 
     // No error.
     ServerPtr server;
     IOAddress addr("127.0.0.1");
-    ASSERT_NO_THROW_LOG(server.reset(new Server(addr, 11645, addr, secret, 0)));
+    TlsContextPtr tls_context;
+    ASSERT_NO_THROW_LOG(server.reset(new Server(addr, 11645, addr, tls_context,
+                                                secret, 0)));
+    ASSERT_TRUE(server);
     servers.push_back(server);
-    ASSERT_NO_THROW_LOG(exchange.reset(new Exchange(msg, 0, servers)));
+    ASSERT_NO_THROW_LOG(exchange = Exchange::create(msg, 0, servers));
 
     // Check exchange.
     ASSERT_TRUE(exchange);
@@ -114,10 +124,13 @@ TEST(TestExchange, sync) {
     EXPECT_EQ(request, msg);
     MessagePtr response = exchange->getResponse();
     EXPECT_FALSE(response);
+
+    // Verify it is an UDP Exchange.
+    EXPECT_TRUE(boost::dynamic_pointer_cast<UdpExchange>(exchange));
 }
 
 /// Test Exchange class.
-class TestExchange : public Exchange {
+class TestExchange : public UdpExchange {
 public:
     /// Constructor.
     ///
@@ -126,59 +139,59 @@ public:
                  unsigned maxretries,
                  const Servers& servers,
                  Exchange::Handler handler)
-        : Exchange(io_service, request, maxretries, servers, handler) {
+        : UdpExchange(io_service, request, maxretries, servers, handler) {
     }
 
     /// Visible members.
     using Exchange::identifier_;
     using Exchange::sync_;
-    using Exchange::started_;
-    using Exchange::terminated_;
     using Exchange::rc_;
-    using Exchange::start_time_;
-    using Exchange::socket_;
-    using Exchange::ep_;
-    using Exchange::timer_;
-    using Exchange::server_;
-    using Exchange::idx_;
     using Exchange::sent_;
     using Exchange::received_;
-    using Exchange::buffer_;
-    using Exchange::size_;
-    using Exchange::retries_;
-    using Exchange::postponed_;
+    using UdpExchange::started_;
+    using UdpExchange::terminated_;
+    using UdpExchange::start_time_;
+    using UdpExchange::socket_;
+    using UdpExchange::ep_;
+    using UdpExchange::timer_;
+    using UdpExchange::server_;
+    using UdpExchange::idx_;
+    using UdpExchange::buffer_;
+    using UdpExchange::size_;
+    using UdpExchange::retries_;
+    using UdpExchange::postponed_;
 
     /// Visible methods.
-    using Exchange::buildRequest;
-    using Exchange::open;
-    using Exchange::receivedHandler;
+    using UdpExchange::buildRequest;
+    using UdpExchange::open;
+    using UdpExchange::receivedHandler;
 };
 
 /// Type of shared pointers to test exchange objets.
 typedef boost::shared_ptr<TestExchange> TestExchangePtr;
 
-
 /// Test fixture for testing code of exchange class.
 class ExchangeTest : public radius::test::AttributeTest {
 public:
-    // Constructor.
+    /// @brief Constructor.
     ExchangeTest()
         : radius::test::AttributeTest (), io_service_(new IOService()),
           code_(0), secret_("foobar"), addr_("127.0.0.1"),
           port_(11460), timeout_(10), deadtime_(0), maxretries_(3),
           called_(false),
           handler_([this] (const ExchangePtr) { called_ = true; }) {
+        RadiusImpl::instance().use_message_authenticator_ = false;
     }
 
-    // Destructor.
+    /// @brief Destructor.
     virtual ~ExchangeTest() {
         if (exchange_) {
             exchange_->shutdown();
         }
         servers_.clear();
 
-        // As a best practice, call any remaining handlers before destroying the
-        // IO context.
+        // As a best practice, call any remaining handlers before
+        // destroying the IO context.
         io_service_->stopAndPoll();
 
         exchange_.reset();
@@ -187,25 +200,26 @@ public:
         servers_.clear();
     }
 
-    // Create request.
+    /// @brief Create request.
     void createRequest() {
         ASSERT_NO_THROW_LOG(request_.reset(new Message(code_, 0, auth_,
                                                        secret_, send_attrs_)));
         ASSERT_TRUE(request_);
     }
 
-    // Add server.
+    /// @brief Add server.
     void addServer() {
         ServerPtr server;
+        TlsContextPtr tls_context;
         ASSERT_NO_THROW_LOG(server.reset(new Server(addr_, port_, addr_,
-                                                    secret_, timeout_,
-                                                    deadtime_)));
+                                                    tls_context, secret_,
+                                                    timeout_, deadtime_)));
         ASSERT_TRUE(server);
         servers_.push_back(server);
         ASSERT_FALSE(servers_.empty());
     }
 
-    // Create exchange.
+    /// @brief Create exchange.
     void createExchange() {
         ASSERT_NO_THROW_LOG(exchange_.reset(new TestExchange(io_service_,
                                                              request_,
@@ -215,49 +229,49 @@ public:
         ASSERT_TRUE(exchange_);
     }
 
-    // IO service.
+    /// @brief IO service.
     IOServicePtr io_service_;
 
-    // Request.
+    /// @brief Request.
     MessagePtr request_;
 
-    // Request code.
+    /// @brief Request code.
     uint8_t code_;
 
-    // Request authenticator.
+    /// @brief Request authenticator.
     vector<uint8_t> auth_;
 
-    // Secret.
+    /// @brief Secret.
     string secret_;
 
-    // Request attributes.
+    /// @brief Request attributes.
     AttributesPtr send_attrs_;
 
-    // Servers.
+    /// @brief Servers.
     Servers servers_;
 
-    // Address.
+    /// @brief Address.
     IOAddress addr_;
 
-    // Port.
+    /// @brief Port.
     uint16_t port_;
 
-    // Timeout.
+    /// @brief Timeout.
     unsigned timeout_;
 
-    // Deadtime.
+    /// @brief Deadtime.
     unsigned deadtime_;
 
-    // Max retries.
+    /// @brief Max retries.
     unsigned maxretries_;
 
-    // Terminated flag: true if and only if the handler was called.
+    /// @brief Terminated flag: true if and only if the handler was called.
     bool called_;
 
-    // Handler.
+    /// @brief Handler.
     Exchange::Handler handler_;
 
-    // Exchange.
+    /// @brief Exchange.
     TestExchangePtr exchange_;
 };
 
@@ -787,7 +801,8 @@ TEST_F(ExchangeTest, openRetryError) {
 // Verify receivedHandler with null exchange.
 TEST_F(ExchangeTest, receivedHandlerNull) {
     auto no_error = boost::system::error_code();
-    EXPECT_THROW_MSG(TestExchange::receivedHandler(ExchangePtr(), no_error, 0), Unexpected,
+    EXPECT_THROW_MSG(TestExchange::receivedHandler(TestExchangePtr(), no_error, 0),
+                     Unexpected,
                      "null exchange in receivedHandler");
 }
 
