@@ -838,6 +838,7 @@ TEST_F(MessageTest, badDecode) {
             0x66, 0x13, 0x3a, 0x40, 0x31, 0x53, 0x61, 0xcd,
             0xf3, 0x29, 0xe2, 0xb5, 0x00, 0x5f, 0x4f, 0x74
         };
+        ASSERT_EQ(AUTH_VECTOR_LEN, auth.size());
         // Correct secret is "bar" so change a bit in it.
         string secret = "Bar";
         MessagePtr message(new Message(buffer, auth, secret));
@@ -1653,4 +1654,514 @@ TEST_F(MessageTest, verboseStatusResponse) {
         << got_attrs->toText() << "\n" << exp_attrs->toText();
 }
 
+// Tests below are based on captures of packets created by FreeRADIUS.
+
+// Verify signed basic Access-Request.
+TEST_F(MessageTest, signedBasicAccessRequest) {
+    MsgCode code = PW_ACCESS_REQUEST;
+    uint8_t id = 0x82;
+    vector<uint8_t> auth = {
+        0xe5, 0x9e, 0xf2, 0x63, 0x38, 0xde, 0x71, 0x9b,
+        0x61, 0xdb, 0x8d, 0x75, 0x4f, 0xb0, 0xcf, 0xc3
+    };
+    ASSERT_EQ(AUTH_VECTOR_LEN, auth.size());
+    string secret = "foo";
+    AttributesPtr attrs(new Attributes());
+    ASSERT_TRUE(attrs);
+
+    vector<uint8_t> zero(AUTH_VECTOR_LEN);
+    attrs->add(Attribute::fromBinary(PW_MESSAGE_AUTHENTICATOR, zero));
+
+    // Create message.
+    MessagePtr request;
+    ASSERT_NO_THROW(request.reset(new Message(code, 0, auth, secret, attrs)));
+    ASSERT_TRUE(request);
+    // Identifier must be set explicitly.
+    request->setIdentifier(id);
+
+    // Encode request.
+    vector<uint8_t> buffer;
+    ASSERT_NO_THROW(buffer = request->encode());
+
+    // Check buffer.
+    uint16_t length = request->getLength();
+    ASSERT_EQ(38, length);
+    vector<uint8_t> got_buffer = request->getBuffer();
+    ASSERT_EQ(buffer.size(), got_buffer.size());
+    EXPECT_TRUE(memcmp(&buffer[0], &got_buffer[0], buffer.size()) == 0);
+
+    // Verify buffer.
+    vector<uint8_t> expected = {
+        0x01,           // Code (Access-Request).
+        0x82,           // Identifier (0x82).
+        0x00, 0x26,     // Length (38).
+        // Authenticator.
+        0xe5, 0x9e, 0xf2, 0x63, 0x38, 0xde, 0x71, 0x9b,
+        0x61, 0xdb, 0x8d, 0x75, 0x4f, 0xb0, 0xcf, 0xc3,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0x8b, 0x9d, 0x0d, 0x3e, 0xc4, 0xf2, 0xc4, 0x4e,
+        0x65, 0xcc, 0xe1, 0x8a, 0x2b, 0x63, 0x97, 0x4b
+    };
+    ASSERT_EQ(38, expected.size());
+    EXPECT_TRUE(memcmp(&expected[0], &buffer[0], buffer.size()) == 0)
+        << str::dumpAsHex(&buffer[0], 38) << "\n"
+        << str::dumpAsHex(&expected[0], 38);
+
+    // Verify the message is signed.
+    auto got_attrs = request->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+}
+
+// Verify signed Access-Request.
+TEST_F(MessageTest, signedAccessRequest) {
+    MsgCode code = PW_ACCESS_REQUEST;
+    uint8_t id = 0x98;
+    vector<uint8_t> auth = {
+        0x22, 0xcb, 0xb3, 0xde, 0x5e, 0xfc, 0x5b, 0xd7,
+        0xab, 0x8f, 0xf6, 0x90, 0x9d, 0x7a, 0xcd, 0x65
+    };
+    ASSERT_EQ(AUTH_VECTOR_LEN, auth.size());
+    string secret = "foo";
+    AttributesPtr attrs(new Attributes());
+    ASSERT_TRUE(attrs);
+
+    vector<uint8_t> zero(AUTH_VECTOR_LEN);
+    attrs->add(Attribute::fromBinary(PW_MESSAGE_AUTHENTICATOR, zero));
+    attrs->add(Attribute::fromString(PW_USER_NAME, "user"));
+    attrs->add(Attribute::fromString(PW_USER_PASSWORD, "secret password"));
+    attrs->add(Attribute::fromInt(PW_NAS_PORT, 1));
+    attrs->add(Attribute::fromIpAddr(PW_NAS_IP_ADDRESS,
+                                      IOAddress("127.0.0.1")));
+
+    // Create message.
+    MessagePtr request;
+    ASSERT_NO_THROW(request.reset(new Message(code, 0, auth, secret, attrs)));
+    ASSERT_TRUE(request);
+    // Identifier must be set explicitly.
+    request->setIdentifier(id);
+
+    // Encode request.
+    vector<uint8_t> buffer;
+    ASSERT_NO_THROW(buffer = request->encode());
+
+    // Check buffer.
+    uint16_t length = request->getLength();
+    ASSERT_EQ(74, length);
+    vector<uint8_t> got_buffer = request->getBuffer();
+    ASSERT_EQ(buffer.size(), got_buffer.size());
+    EXPECT_TRUE(memcmp(&buffer[0], &got_buffer[0], buffer.size()) == 0);
+
+    // Verify buffer.
+    vector<uint8_t> expected = {
+        0x01,           // Code (Access-Request).
+        0x98,           // Identifier (0x98).
+        0x00, 0x4a,     // Length (74).
+        // Authenticator.
+        0x22, 0xcb, 0xb3, 0xde, 0x5e, 0xfc, 0x5b, 0xd7,
+        0xab, 0x8f, 0xf6, 0x90, 0x9d, 0x7a, 0xcd, 0x65,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0x98, 0x19, 0xa8, 0xb5, 0x59, 0xc6, 0xd3, 0x38,
+        0xf3, 0x4b, 0x19, 0xe3, 0xa5, 0x2b, 0xdb, 0x84,
+        // User-Name ("user").
+        0x01, 0x06, 0x75, 0x73, 0x65, 0x72,
+        // User-Password ("secret password") padded and ciphered.
+        0x02, 0x12,
+        0x00, 0x3b, 0x46, 0x67, 0xf5, 0xcd, 0x15, 0xca,
+        0x53, 0x30, 0x9e, 0xed, 0x93, 0x49, 0x14, 0x68,
+        // NAS-Port (1).
+        0x05, 0x06, 0x00, 0x00, 0x00, 0x01,
+        // NAS-IP-Address (127.0.0.1).
+        0x04, 0x06, 0x7f, 0x00, 0x00, 0x01
+    };
+    ASSERT_EQ(74, expected.size());
+    EXPECT_TRUE(memcmp(&expected[0], &buffer[0], buffer.size()) == 0)
+        << str::dumpAsHex(&buffer[0], 74) << "\n"
+        << str::dumpAsHex(&expected[0], 74);
+
+    // Verify the message is signed.
+    auto got_attrs = request->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+
+    // And correctly decoded.
+    auto got_attr = got_attrs->get(PW_USER_NAME);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ("user", got_attr->toString());
+    got_attr = got_attrs->get(PW_USER_PASSWORD);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ("secret password", got_attr->toString());
+    got_attr = got_attrs->get(PW_NAS_PORT);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ(1, got_attr->toInt());
+    got_attr = got_attrs->get(PW_NAS_IP_ADDRESS);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ("127.0.0.1", got_attr->toIpAddr().toText());
+}
+
+// Verify signed basic Accounting-Request.
+// Note this is critical as not specified in a standard and to use
+// a random request authenticator would be another solution...
+TEST_F(MessageTest, signedBasicAccountingRequest) {
+    MsgCode code = PW_ACCOUNTING_REQUEST;
+    uint8_t id = 0x62;
+    string secret = "bar";
+    AttributesPtr attrs(new Attributes());
+    ASSERT_TRUE(attrs);
+
+    vector<uint8_t> zero(AUTH_VECTOR_LEN);
+    attrs->add(Attribute::fromBinary(PW_MESSAGE_AUTHENTICATOR, zero));
+
+    // Create message.
+    MessagePtr request;
+    ASSERT_NO_THROW(request.reset(new Message(code, 0, Message::ZERO_AUTH(),
+                                              secret, attrs)));
+    ASSERT_TRUE(request);
+    // Identifier must be set explicitly.
+    request->setIdentifier(id);
+
+    // Encode request.
+    vector<uint8_t> buffer;
+    ASSERT_NO_THROW(buffer = request->encode());
+
+    // Check buffer.
+    uint16_t length = request->getLength();
+    ASSERT_EQ(38, length);
+    vector<uint8_t> got_buffer = request->getBuffer();
+    ASSERT_EQ(buffer.size(), got_buffer.size());
+    EXPECT_TRUE(memcmp(&buffer[0], &got_buffer[0], buffer.size()) == 0);
+
+    // Verify buffer.
+    vector<uint8_t> expected = {
+        0x04,           // Code (Accounting-Request).
+        0x62,           // Identifier (0x62).
+        0x00, 0x26,     // Length (38).
+        // Authenticator.
+        0xfd, 0xa2, 0xbb, 0xa0, 0x39, 0x58, 0x71, 0x00,
+        0x65, 0x9a, 0x1b, 0x74, 0x50, 0xa0, 0xeb, 0x02,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0x60, 0xda, 0x7f, 0x9f, 0x0a, 0x11, 0xfe, 0x56,
+        0xfc, 0x46, 0xcf, 0xef, 0x9c, 0x9c, 0x33, 0x60
+    };
+    ASSERT_EQ(38, expected.size());
+    EXPECT_TRUE(memcmp(&expected[0], &buffer[0], buffer.size()) == 0)
+        << str::dumpAsHex(&buffer[0], 38) << "\n"
+        << str::dumpAsHex(&expected[0], 38);
+
+    // Verify the message is signed.
+    auto got_attrs = request->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+}
+
+// Verify signed Accounting-Request.
+TEST_F(MessageTest, signedAccountingRequest) {
+    MsgCode code = PW_ACCOUNTING_REQUEST;
+    uint8_t id = 0x40;
+    string secret = "bar";
+    AttributesPtr attrs(new Attributes());
+    ASSERT_TRUE(attrs);
+
+    vector<uint8_t> zero(AUTH_VECTOR_LEN);
+    attrs->add(Attribute::fromBinary(PW_MESSAGE_AUTHENTICATOR, zero));
+    attrs->add(Attribute::fromString(PW_USER_NAME, "user"));
+    attrs->add(Attribute::fromInt(PW_NAS_PORT, 1));
+    attrs->add(Attribute::fromInt(PW_ACCT_DELAY_TIME, 0));
+    attrs->add(Attribute::fromIpAddr(PW_NAS_IP_ADDRESS,
+                                     IOAddress("127.0.0.1")));
+
+    // Create message.
+    MessagePtr request;
+    ASSERT_NO_THROW(request.reset(new Message(code, 0, Message::ZERO_AUTH(),
+                                              secret, attrs)));
+    ASSERT_TRUE(request);
+    // Identifier must be set explicitly.
+    request->setIdentifier(id);
+
+    // Encode request.
+    vector<uint8_t> buffer;
+    ASSERT_NO_THROW(buffer = request->encode());
+
+    // Check buffer.
+    uint16_t length = request->getLength();
+    ASSERT_EQ(62, length);
+    vector<uint8_t> got_buffer = request->getBuffer();
+    ASSERT_EQ(buffer.size(), got_buffer.size());
+    EXPECT_TRUE(memcmp(&buffer[0], &got_buffer[0], buffer.size()) == 0);
+
+    // Verify buffer.
+    vector<uint8_t> expected = {
+        0x04,           // Code (Accounting-Request).
+        0x40,           // Identifier (0x40).
+        0x00, 0x3e,     // Length (62).
+        // Authenticator.
+        0x16, 0x54, 0xa1, 0x67, 0x6c, 0x6c, 0xc6, 0xf2,
+        0xaf, 0x26, 0x0e, 0xef, 0xa3, 0x23, 0xe9, 0x5b,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0xd7, 0x26, 0xda, 0x56, 0xd6, 0xd5, 0x3f, 0xc3,
+        0x20, 0x2f, 0x8a, 0x60, 0x89, 0xb5, 0x0d, 0x3d,
+        // User-Name ("user").
+        0x01, 0x06, 0x75, 0x73, 0x65, 0x72,
+        // NAS-Port (1).
+        0x05, 0x06, 0x00, 0x00, 0x00, 0x01,
+        // Acct-Delay-Time (0).
+        0x29, 0x06, 0x00, 0x00, 0x00, 0x00,
+        // NAS-IP-Address (127.0.0.1).
+        0x04, 0x06, 0x7f, 0x00, 0x00, 0x01
+    };
+    ASSERT_EQ(62, expected.size());
+    EXPECT_TRUE(memcmp(&expected[0], &buffer[0], buffer.size()) == 0)
+        << str::dumpAsHex(&buffer[0], 62) << "\n"
+        << str::dumpAsHex(&expected[0], 62);
+
+    // Verify the message is signed.
+    auto got_attrs = request->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+
+    // And correctly decoded.
+    auto got_attr = got_attrs->get(PW_USER_NAME);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ("user", got_attr->toString());
+    got_attr = got_attrs->get(PW_NAS_PORT);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ(1, got_attr->toInt());
+    got_attr = got_attrs->get(PW_ACCT_DELAY_TIME);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ(0, got_attr->toInt());
+    got_attr = got_attrs->get(PW_NAS_IP_ADDRESS);
+    ASSERT_TRUE(got_attr);
+    EXPECT_EQ("127.0.0.1", got_attr->toIpAddr().toText());
+}
+
+// Verify signed Access-Accept response to signed Status-Server.
+TEST_F(MessageTest, signedAuthStatusResponse) {
+    MsgCode code = PW_STATUS_SERVER;
+    uint8_t id = 0x1f;
+    vector<uint8_t> auth = {
+        0x38, 0x85, 0x4e, 0x26, 0x26, 0x06, 0xaf, 0x84,
+        0x57, 0x77, 0xe0, 0x2c, 0xfc, 0x76, 0xd5, 0x84
+    };
+    ASSERT_EQ(AUTH_VECTOR_LEN, auth.size());
+    string secret = "foo";
+    AttributesPtr attrs(new Attributes());
+    ASSERT_TRUE(attrs);
+
+    vector<uint8_t> zero(AUTH_VECTOR_LEN);
+    attrs->add(Attribute::fromBinary(PW_MESSAGE_AUTHENTICATOR, zero));
+
+    // Create message.
+    MessagePtr request;
+    ASSERT_NO_THROW(request.reset(new Message(code, 0, auth, secret, attrs)));
+    ASSERT_TRUE(request);
+    // Identifier must be set explicitly.
+    request->setIdentifier(id);
+
+    // Encode request.
+    vector<uint8_t> buffer;
+    ASSERT_NO_THROW(buffer = request->encode());
+
+    // Check buffer.
+    uint16_t length = request->getLength();
+    ASSERT_EQ(38, length);
+    vector<uint8_t> got_buffer = request->getBuffer();
+    ASSERT_EQ(buffer.size(), got_buffer.size());
+    EXPECT_TRUE(memcmp(&buffer[0], &got_buffer[0], buffer.size()) == 0);
+
+    // Verify buffer.
+    vector<uint8_t> expected = {
+        0x0c,           // Code (Status-Server).
+        0x1f,           // Identifier (0x1f).
+        0x00, 0x26,     // Length (38).
+        // Authenticator.
+        0x38, 0x85, 0x4e, 0x26, 0x26, 0x06, 0xaf, 0x84,
+        0x57, 0x77, 0xe0, 0x2c, 0xfc, 0x76, 0xd5, 0x84,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0x15, 0xa5, 0x8a, 0x0b, 0xaa, 0x3b, 0x5f, 0x6d,
+        0xa0, 0xbd, 0xfc, 0xa6, 0xde, 0x60, 0xf9, 0x0f
+    };
+    ASSERT_EQ(38, expected.size());
+    EXPECT_TRUE(memcmp(&expected[0], &buffer[0], buffer.size()) == 0)
+        << str::dumpAsHex(&buffer[0], 38) << "\n"
+        << str::dumpAsHex(&expected[0], 38);
+
+    // Verify the message is signed.
+    auto got_attrs = request->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+
+    // Create message (response).
+    vector<uint8_t> dumped = {
+        0x02,           // Code (Access-Accept).
+        0x1f,           // Identifier (0x1f).
+        0x00, 0x26,     // Length (38).
+        // Authenticator.
+        0x9c, 0x98, 0x42, 0x18, 0xf7, 0xfb, 0xfd, 0x71,
+        0xa1, 0x25, 0x4c, 0x69, 0x9b, 0x95, 0xfd, 0xbc,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0xb9, 0x29, 0x81, 0xa1, 0xa9, 0x21, 0xd6, 0x24,
+        0xc4, 0x58, 0x6a, 0xd0, 0x44, 0x0a, 0x6f, 0xb7
+    };
+    MessagePtr response;
+    ASSERT_NO_THROW(response.reset(new Message(dumped, auth, secret)));
+    ASSERT_TRUE(response);
+
+    // Decode response.
+    ASSERT_NO_THROW(response->decode());
+
+    // Verify the response is signed.
+    got_attrs = response->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+}
+
+// Verify signed Accounting-Response response to signed Status-Server.
+TEST_F(MessageTest, signedAcctStatusResponse) {
+    MsgCode code = PW_STATUS_SERVER;
+    uint8_t id = 0x1b;
+    vector<uint8_t> auth = {
+        0x7d, 0x23, 0xd2, 0xf4, 0x04, 0x86, 0x57, 0x17,
+        0xf9, 0xd2, 0xa8, 0x63, 0x36, 0xc7, 0xc1, 0xfc
+    };
+    ASSERT_EQ(AUTH_VECTOR_LEN, auth.size());
+    string secret = "foo";
+    AttributesPtr attrs(new Attributes());
+    ASSERT_TRUE(attrs);
+
+    vector<uint8_t> zero(AUTH_VECTOR_LEN);
+    attrs->add(Attribute::fromBinary(PW_MESSAGE_AUTHENTICATOR, zero));
+
+    // Create message.
+    MessagePtr request;
+    ASSERT_NO_THROW(request.reset(new Message(code, 0, auth, secret, attrs)));
+    ASSERT_TRUE(request);
+
+    // Identifier must be set explicitly.
+    request->setIdentifier(id);
+
+    // Encode request.
+    vector<uint8_t> buffer;
+    ASSERT_NO_THROW(buffer = request->encode());
+
+    // Check buffer.
+    uint16_t length = request->getLength();
+    ASSERT_EQ(38, length);
+    vector<uint8_t> got_buffer = request->getBuffer();
+    ASSERT_EQ(buffer.size(), got_buffer.size());
+    EXPECT_TRUE(memcmp(&buffer[0], &got_buffer[0], buffer.size()) == 0);
+
+    // Verify buffer.
+    vector<uint8_t> expected = {
+        0x0c,           // Code (Status-Server).
+        0x1b,           // Identifier (0x1b).
+        0x00, 0x26,     // Length (38).
+        // Authenticator.
+        0x7d, 0x23, 0xd2, 0xf4, 0x04, 0x86, 0x57, 0x17,
+        0xf9, 0xd2, 0xa8, 0x63, 0x36, 0xc7, 0xc1, 0xfc,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0xab, 0x13, 0x50, 0x87, 0x20, 0xbd, 0xf6, 0xe1,
+        0xf1, 0x89, 0x02, 0x81, 0xf7, 0xeb, 0xac, 0x1d
+    };
+    ASSERT_EQ(38, expected.size());
+    EXPECT_TRUE(memcmp(&expected[0], &buffer[0], buffer.size()) == 0)
+        << str::dumpAsHex(&buffer[0], 38) << "\n"
+        << str::dumpAsHex(&expected[0], 38);
+
+    // Verify the request is signed.
+    auto got_attrs = request->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+
+    // Create message (response).
+    vector<uint8_t> dumped = {
+        0x05,           // Code (Accounting-Response).
+        0x1b,           // Identifier (0x1b).
+        0x00, 0x26,     // Length (38).
+        // Authenticator.
+        0xc0, 0x6f, 0x9d, 0xd2, 0x03, 0x0b, 0xea, 0x0b,
+        0xa4, 0x30, 0x0d, 0x5e, 0x36, 0xe5, 0x15, 0x4a,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0xb2, 0xa7, 0x57, 0x49, 0xa4, 0x11, 0x2b, 0x6a,
+        0xf9, 0xaa, 0x9b, 0x31, 0x73, 0xdb, 0xdb, 0xe8
+    };
+    MessagePtr response;
+    ASSERT_NO_THROW(response.reset(new Message(dumped, auth, secret)));
+    ASSERT_TRUE(response);
+
+    // Decode response.
+    ASSERT_NO_THROW(response->decode());
+
+    // Verify the response is signed.
+    got_attrs = response->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+}
+
+// Verify signed Access-Reject (only used message not yet tested).
+TEST_F(MessageTest, signedAccessReject) {
+    MsgCode code = PW_ACCESS_REJECT;
+    uint8_t id = 0x4d;
+    // Taken from the Access-Request.
+    vector<uint8_t> auth = {
+        0x3d, 0x8a, 0x4a, 0x91, 0x27, 0x4f, 0x48, 0x84,
+        0x52, 0x25, 0x48, 0xa9, 0x04, 0xca, 0x56, 0x30
+    };
+    ASSERT_EQ(AUTH_VECTOR_LEN, auth.size());
+    string secret = "foo";
+    AttributesPtr attrs(new Attributes());
+    ASSERT_TRUE(attrs);
+
+    vector<uint8_t> zero(AUTH_VECTOR_LEN);
+    attrs->add(Attribute::fromBinary(PW_MESSAGE_AUTHENTICATOR, zero));
+
+    // Create message.
+    MessagePtr reject;
+    ASSERT_NO_THROW(reject.reset(new Message(code, 0, auth, secret, attrs)));
+    ASSERT_TRUE(reject);
+    // Identifier must be set explicitly.
+    reject->setIdentifier(id);
+
+    // Encode reject.
+    vector<uint8_t> buffer;
+    ASSERT_NO_THROW(buffer = reject->encode());
+
+    // Check buffer.
+    uint16_t length = reject->getLength();
+    ASSERT_EQ(38, length);
+    vector<uint8_t> got_buffer = reject->getBuffer();
+    ASSERT_EQ(buffer.size(), got_buffer.size());
+    EXPECT_TRUE(memcmp(&buffer[0], &got_buffer[0], buffer.size()) == 0);
+
+    // Verify buffer.
+    vector<uint8_t> expected = {
+        0x03,           // Code (Access-Reject).
+        0x4d,           // Identifier (0x4d).
+        0x00, 0x26,     // Length (38).
+        // Authenticator.
+        0xc3, 0x7c, 0xe0, 0xff, 0x30, 0x5a, 0x03, 0xc7,
+        0xd0, 0xb0, 0x81, 0x76, 0x8f, 0x9c, 0xcb, 0x5e,
+        // Message-Authenticator.
+        0x50, 0x12,
+        0x13, 0x40, 0x2f, 0xae, 0x82, 0x71, 0x59, 0x80,
+        0xff, 0xfc, 0xb7, 0x9a, 0xf8, 0xbf, 0x97, 0x89
+    };
+    ASSERT_EQ(38, expected.size());
+    EXPECT_TRUE(memcmp(&expected[0], &buffer[0], buffer.size()) == 0)
+        << str::dumpAsHex(&buffer[0], 38) << "\n"
+        << str::dumpAsHex(&expected[0], 38);
+
+    // Verify the message is signed.
+    auto got_attrs = reject->getAttributes();
+    ASSERT_TRUE(got_attrs);
+    EXPECT_EQ(1, got_attrs->count(PW_MESSAGE_AUTHENTICATOR));
+}
+
 } // end of anonymous namespace
