@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2025 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2020-2026 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -54,7 +54,9 @@ const char* CONFIGS[] = {
 "            \"port\": 11813,\n"
 "            \"secret\": \"bar\" } ] },\n"
 "    \"retries\": 0,\n"
-"    \"timeout\": 1 }\n",
+"    \"timeout\": 1,\n"
+"    \"use-message-authenticator\": false\n"
+"}\n",
     // CONFIGURATION 1
 "{\n"
 "    \"access\": {\n"
@@ -76,7 +78,9 @@ const char* CONFIGS[] = {
 "            \"port\": 11813,\n"
 "            \"secret\": \"bar\" } ] },\n"
 "    \"retries\": 0,\n"
-"    \"timeout\": 1 }\n"
+"    \"timeout\": 1,\n"
+"    \"use-message-authenticator\": false\n"
+"}\n"
 };
 
 /// @brief Class for callbacks.
@@ -201,6 +205,7 @@ public:
     void doneAcct(int result);
 
     /// @brief Run a request handler i.e. the client part.
+    ///
     /// The server side is simulated by the individual test code.
     void run();
 
@@ -1185,7 +1190,6 @@ TEST_F(RequestTest, shortAcct) {
     size = AUTH_HDR_LEN;                       // header (no attributes).
     send_buffer_.resize(size);
     send_buffer_[0] = PW_ACCOUNTING_RESPONSE;  // Access-Accept.
-    // There are a lot of ways to get an error including this one.
     send_buffer_[1] = receive_buffer_[1];      // Copy id.
     send_buffer_[2] = size >> 8;               // Length
     send_buffer_[3] = size & 0xff;
