@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2025 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2026 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,6 +9,7 @@
 #include <dhcp/option6_client_fqdn.h>
 #include <dhcpsrv/d2_client_mgr.h>
 #include <testutils/test_to_element.h>
+#include <testutils/gtest_utils.h>
 #include <exceptions/exceptions.h>
 #include <util/str.h>
 
@@ -113,8 +114,11 @@ TEST(D2ClientConfigTest, constructorsAndAccessors) {
     EXPECT_EQ(d2_client_config->getContext()->str(), user_context);
 
     // Verify that toText called by << operator doesn't bomb.
-    ASSERT_NO_THROW(std::cout << "toText test:" << std::endl <<
-                    *d2_client_config << std::endl);
+    stringstream stream;
+    ASSERT_NO_THROW(stream << "toText test: " << *d2_client_config);
+    EXPECT_EQ(stream.str(), "toText test: enable_updates: yes, server-ip: 127.0.0.1, server-port: "
+                            "477, sender-ip: 127.0.0.1, sender-port: 478, max-queue-size: 2048, "
+                            "ncr-protocol: UDP, ncr-format: JSON");
 
     // Verify what toElement returns.
     std::string expected = "{\n"
@@ -627,6 +631,10 @@ TEST_F(D2ClientMgrParamsTest, qualifyName) {
     qualified_name = mgr.qualifyName(partial_name, *ddns_params_, do_not_dot);
     EXPECT_EQ("somehost.suffix.com", qualified_name);
 
+    // Verify that an empty name throws.
+    partial_name = "";
+    ASSERT_THROW(mgr.qualifyName(partial_name, *ddns_params_, do_not_dot), BadValue);
+
     // Verify that an empty suffix and false flag, does not change the name
     subnet_->setDdnsQualifyingSuffix("");
     partial_name = "somehost";
@@ -639,13 +647,8 @@ TEST_F(D2ClientMgrParamsTest, qualifyName) {
     qualified_name = mgr.qualifyName(partial_name, *ddns_params_, do_dot);
     EXPECT_EQ("somehost.hasdot.com.", qualified_name);
 
-    // Verify that the qualifying suffix gets appended without an
-    // extraneous dot when partial_name ends with a "."
-    qualified_name = mgr.qualifyName("somehost.", *ddns_params_, do_dot);
-    EXPECT_EQ("somehost.hasdot.com.", qualified_name);
-
-    // Verify that a name with a trailing dot does not get an extraneous
-    // dot when the suffix is blank
+    // Verify that the qualifying suffix does not get appended nor an
+    // extraneous dot added to a name with a trailing dot.
     subnet_->setDdnsQualifyingSuffix("");
     qualified_name = mgr.qualifyName("somehost.", *ddns_params_, do_dot);
     EXPECT_EQ("somehost.", qualified_name);
@@ -664,7 +667,6 @@ TEST_F(D2ClientMgrParamsTest, qualifyName) {
     // suffix is blank and trailing dot is false
     qualified_name = mgr.qualifyName("somehost.", *ddns_params_, do_not_dot);
     EXPECT_EQ("somehost", qualified_name);
-
 }
 
 /// @brief Tests the qualifyName method's ability to avoid duplicating
@@ -802,6 +804,12 @@ TEST_F(D2ClientMgrParamsTest, adjustDomainNameV4) {
             "myhost.example.com.", Option4ClientFqdn::FULL,
             "myhost.example.com.", Option4ClientFqdn::FULL
         },
+        {
+            "RCM_NEVER #4, partial client name with trailing .",
+            D2ClientConfig::RCM_NEVER,
+            "myhost.", Option4ClientFqdn::PARTIAL,
+            "myhost.suffix.com.", Option4ClientFqdn::FULL
+        },
         {
             "RCM_ALWAYS #1, empty client name",
             D2ClientConfig::RCM_ALWAYS,
@@ -917,6 +925,12 @@ TEST_F(D2ClientMgrParamsTest, adjustDomainNameV6) {
             "myhost.example.com.", Option6ClientFqdn::FULL,
             "myhost.example.com.", Option6ClientFqdn::FULL
         },
+        {
+            "RCM_NEVER #4, partial client name with trailing .",
+            D2ClientConfig::RCM_NEVER,
+            "myhost.", Option6ClientFqdn::PARTIAL,
+            "myhost.suffix.com.", Option6ClientFqdn::FULL
+        },
         {
             "RCM_ALWAYS #1, empty client name",
             D2ClientConfig::RCM_ALWAYS,
@@ -1251,4 +1265,41 @@ TEST_F(D2ClientMgrParamsTest, sanitizeFqdnV6) {
     }
 }
 
+/// @brief Tests adjustDomainName template method with Option4ClientFqdn
+/// when sanitizing scrubs input name empty.
+TEST_F(D2ClientMgrParamsTest, adjustDomainNameV4ScrubbedEmpty) {
+    D2ClientMgr mgr;
+
+    // Create enabled configuration
+    subnet_->setDdnsSendUpdates(false);
+    subnet_->setDdnsQualifyingSuffix("suffix.com");
+    subnet_->setHostnameCharSet("[^A-Za-z0-9.-]");
+    subnet_->setHostnameCharReplacement("");
+
+    Option4ClientFqdn request(0, Option4ClientFqdn::RCODE_CLIENT(),
+                              "___", Option4ClientFqdn::FULL);
+
+    Option4ClientFqdn response(request);
+    ASSERT_THROW_MSG(mgr.adjustDomainName<Option4ClientFqdn>(request, response, *ddns_params_),
+                     FQDNScrubbedEmpty, "___.");
+}
+
+/// @brief Tests adjustDomainName template method with Option4ClientFqdn
+/// when sanitizing scrubs input name empty.
+TEST_F(D2ClientMgrParamsTest, adjustDomainNameV6ScrubbedEmpty) {
+    D2ClientMgr mgr;
+
+    // Create enabled configuration
+    subnet_->setDdnsSendUpdates(false);
+    subnet_->setDdnsQualifyingSuffix("suffix.com");
+    subnet_->setHostnameCharSet("[^A-Za-z0-9.-]");
+    subnet_->setHostnameCharReplacement("");
+
+    Option6ClientFqdn request(0, "___", Option6ClientFqdn::FULL);
+
+    Option6ClientFqdn response(request);
+    ASSERT_THROW_MSG(mgr.adjustDomainName<Option6ClientFqdn>(request, response, *ddns_params_),
+                     FQDNScrubbedEmpty, "___.");
+}
+
 } // end of anonymous namespace
