@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2025 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2026 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -127,6 +127,21 @@ TEST_F(UMaskUtilTest, umask077) {
     EXPECT_EQ(S_IRWXG | S_IRWXO, umask(0));
 }
 
+/// @brief Check RelaxUmask.
+TEST_F(UMaskUtilTest, relaxUmask) {
+    static_cast<void>(umask(0));
+    {
+        RelaxUmask ru;
+        EXPECT_EQ(S_IRWXO, umask(0));
+    }
+    EXPECT_EQ(0, umask(S_IWGRP | S_IRWXO));
+    {
+        RelaxUmask ru;
+        EXPECT_EQ(S_IRWXO, umask(0));
+    }
+    EXPECT_EQ(S_IWGRP | S_IRWXO, umask(0));
+}
+
 /// @brief Check that the components are split correctly.
 TEST(PathTest, components) {
     // Complete name
